Type: Package
Package: desplot
Title: Plotting Field Plans for Agricultural Experiments
Version: 1.11
Authors@R: c(
    person("Kevin", "Wright", , "kw.stat@gmail.com", role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0000-0002-0617-8673")),
    person("Paul", "Schmidt", role = c("aut"))
  )
Description: A function for plotting maps of agricultural field
    experiments that are laid out in grids.  See Ryder (1981)
    <doi:10.1017/S0014479700011601>.
License: MIT + file LICENSE
URL: https://kwstat.github.io/desplot/,
        http://kwstat.github.io/desplot/
BugReports: https://github.com/kwstat/desplot/issues
Imports: ggplot2, grid, lattice, reshape2, rlang
Suggests: agridat, knitr, rmarkdown, testthat
VignetteBuilder: knitr
Encoding: UTF-8
Language: en-US
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-08-07 20:16:44 UTC; wrightkevi
Author: Kevin Wright [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-0617-8673>),
  Paul Schmidt [aut]
Maintainer: Kevin Wright <kw.stat@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-21 12:00:08 UTC
Built: R 4.6.0; ; 2026-08-21 14:16:21 UTC; unix

Package: desplot
Title: Plotting Field Plans for Agricultural Experiments
Version: 1.10
Authors@R: person("Kevin","Wright", email="kw.stat@gmail.com", comment=c(ORCID = "0000-0002-0617-8673"), role=c("aut","cre"))
Type: Package
Description: A function for plotting maps of agricultural field experiments that are laid out in grids.  See Ryder (1981) <doi:10.1017/S0014479700011601>.
Imports: ggplot2, grid, lattice, reshape2, rlang
Suggests: agridat, knitr, rmarkdown, testthat
License: GPL-3
URL: https://kwstat.github.io/desplot/
BugReports: https://github.com/kwstat/desplot/issues
VignetteBuilder: knitr
RoxygenNote: 7.2.2
Encoding: UTF-8
Language: en-US
NeedsCompilation: no
Packaged: 2023-03-09 22:45:26 UTC; wrightkevi
Author: Kevin Wright [aut, cre] (<https://orcid.org/0000-0002-0617-8673>)
Maintainer: Kevin Wright <kw.stat@gmail.com>
Repository: CRAN
Date/Publication: 2023-03-09 23:20:06 UTC
Built: R 4.2.0; ; 2023-07-11 01:15:47 UTC; unix

# Generated by roxygen2: do not edit by hand

export(RedGrayBlue)
export(StatTileBorder)
export(desplot)
export(geom_tileborder)
export(ggdesplot)
export(panel.outlinelevelplot)
import(ggplot2)
import(grid)
import(lattice)
importFrom(ggplot2,Stat)
importFrom(ggplot2,ggproto)
importFrom(reshape2,acast)
importFrom(reshape2,melt)
importFrom(rlang,.data)
importFrom(stats,as.formula)
importFrom(stats,formula)
importFrom(stats,median)
importFrom(stats,update)

## ----setup------------------------------------------------------------------------------
library("knitr")
knitr::opts_chunk$set(fig.align="center", fig.width=6, fig.height=6)
options(width=90)

## ----yates------------------------------------------------------------------------------
library(agridat)
library(desplot)
data(yates.oats)
# Older versions of agridat used x/y here instead of col/row
if(is.element("x",names(yates.oats)))
   yates.oats <- transform(yates.oats, col=x, row=y)
desplot(yates.oats, block ~ col+row,
        col=nitro, text=gen, cex=1, out1=block,
        out2=gen, out2.gpar=list(col = "gray50", lwd = 1, lty = 1))

## ----ryder------------------------------------------------------------------------------
library(agridat)
library(desplot)
data(ryder.groundnut)
gnut <- ryder.groundnut
m1 <- lm(dry ~ block + gen, gnut) # Standard RCB model
gnut$res <- resid(m1)
desplot(gnut, res ~ col + row, text=gen, cex=1,
        main="ryder.groundnut residuals from RCB model")

# ggdesplot.R

# Currently the "outline" and "text" do not have guides.
# Perhaps this can be used for multiple scales:
# https://github.com/eliocamp/ggnewscale

if(0){
  
  # multiple legends
  # https://stackoverflow.com/questions/18394391/r-custom-legend-for-multiple-layer-ggplot#18395012
  
  # perfect example for desplot, but no facets
  # https://stackoverflow.com/questions/25704567/overlay-ggplot-grouped-tiles-with-polygon-border-depending-on-extra-factor
  
  # https://stackoverflow.com/questions/36156387/how-to-make-a-custom-ggplot2-geom-with-multiple-geometries
  
  # https://stackoverflow.com/questions/11846295/how-to-add-different-lines-for-facets#11847210
  
}

if(0){
  libs(agridat)
  ggdesplot(besag.met, ~ col*row|county)
  ggdesplot(besag.met, ~ col*row|county, col=block)
  ggdesplot(besag.met, ~ col*row|county, num=block)
  ggdesplot(besag.met,  ~ col*row|county, text=block)
  
  ggdesplot(besag.met, rep ~ col*row|county)
  ggdesplot(besag.met, rep ~ col*row|county, col=block, cex=.8)
  ggdesplot(besag.met, rep ~ col*row|county, num=block, cex=.8)
  ggdesplot(besag.met, rep ~ col*row|county, text=block, cex=.8)
  
  ggdesplot(besag.met, yield ~ col*row|county, out1=rep)
  ggdesplot(besag.met, yield ~ col*row|county, out2=block)
  ggdesplot(besag.met, yield ~ col*row|county, out1=rep, out2=block)
  
  ggdesplot(besag.met, rep ~ col*row|county, ticks=TRUE)
  ggdesplot(besag.met, rep ~ col*row|county, ticks=TRUE, flip=TRUE)
                      
  ggdesplot(besag.met, rep ~ col*row|county, out1=rep, ticks=TRUE,
            main="besag.met", xlab="column", ylab="row")

  ggdesplot(besag.met, rep ~ col*row|county, out1=rep, show.key=FALSE)
  ##                   col.regions=RedGrayBlue, col.text=NULL, text.levels=NULL,
  ##                   out1.gpar=list(col="black", lwd=3),
  ##                   out2.gpar=list(col="yellow", lwd=1, lty=1),
  ##                   at, midpoint="median",
  ##                   shorten='abb',
  ##                   key.cex, # left legend cex
  ##                   strip.cex=.75, 
  
}

# Note: To avoid a similar note from the CMD check about .data,
# use #' @importFrom rlang .data

#' @import ggplot2
#' @importFrom stats as.formula formula median
#' @importFrom rlang .data
#' @export
#' @rdname desplot
ggdesplot <- function(data,
                      form=formula(NULL ~ x + y),
                      num=NULL, num.string=NULL,
                      col=NULL, col.string=NULL,
                      text=NULL, text.string=NULL,
                      out1=NULL, out1.string=NULL,
                      out2=NULL, out2.string=NULL,
                      dq=NULL, dq.string=NULL,
                      col.regions=RedGrayBlue, col.text=NULL, text.levels=NULL,
                      out1.gpar=list(col="black", lwd=3),
                      out2.gpar=list(col="yellow", lwd=1, lty=1),
                      at, midpoint="median",
                      ticks=FALSE, panel.border=TRUE, flip=FALSE,
                      main=NULL, xlab, ylab,
                      shorten='abb',
                      show.key=TRUE,
                      key.cex, # left legend cex
                      cex=.4, # cell cex
                      strip.cex=.75,
                      aspect=NULL, # aspect ratio for true-scale field maps
                      subset=TRUE, gg=FALSE, ...){

  # subset, based on subset() function
  ix <- if (missing(subset)) 
    rep_len(TRUE, nrow(data))
  else {
    e <- substitute(subset)
    ix <- eval(e, data, parent.frame())
    if (!is.logical(ix)) 
      stop("'subset' must be logical")
    ix & !is.na(ix)
  }
  data <- data[ix, ]
  data <- droplevels(data) # In case the user called with subset(obj, ...)
  
  
  # Using 'at' overrides 'midpoint'
  if(!missing(at) && !is.null(midpoint))
    midpoint <- NULL

  if(!missing(at) && is.vector(col.regions) &&
       ( length(at) !=  length(col.regions)+1 ) )
    stop("Length of 'at' must be 1 more than length of 'col.regions'\n")
  
  # Assume num.string contains the name of a column in data. 
  # If num.string is NULL, then get its value by converting 'num' 
  # from a bare name to a string.  We MUST do this here
  # so that if we switch from desplot to ggdesplot, we can pass 
  # arguments as strings.  
  mc <- as.list(match.call())
  
  if(is.null(num.string)){
    if("num" %in% names(mc)) { # user did supply argument
      if(!is.character(mc$num) & !is.null(mc$num)) { # it is a bare name
        num.string <- deparse(substitute(num)) # evaluate to string
      }
    }
  }
  
  if(is.null(col.string)){
    if("col" %in% names(mc)) {
      if(!is.character(mc$col) & !is.null(mc$col)) {
        col.string <- deparse(substitute(col))
      }
    }
  }
  
  if(is.null(text.string)){
    if("text" %in% names(mc)) {
      if(!is.character(mc$text) & !is.null(mc$text)) {
        text.string <- deparse(substitute(text))
      }
    }
  }
  
  if(is.null(out1.string)){
    if("out1" %in% names(mc)) {
      if(!is.character(mc$out1) & !is.null(mc$out1)) {
        out1.string <- deparse(substitute(out1))
      }
    }
  }
  
  if(is.null(out2.string)){
    if("out2" %in% names(mc)) {
      if(!is.character(mc$out2) & !is.null(mc$out2)) {
        out2.string <- deparse(substitute(out2))
      }
    }
  }
  
  if(is.null(dq.string)){
    if("dq" %in% names(mc)) {
      if(!is.character(mc$dq) & !is.null(mc$dq)) {
        dq.string <- deparse(substitute(dq))
      }
    }
  }
  
  dn <- names(data)
  checkvars <- function(x, dn){
    if(!is.null(x) && !is.element(x, dn))
      stop("Could not find '", x,"' in the data frame.")
  }
  checkvars(num.string, dn)
  checkvars(col.string, dn)
  checkvars(text.string, dn)
  checkvars(out1.string, dn)
  checkvars(out2.string, dn)
  checkvars(dq.string, dn)
  
  has.num <- !is.null(num.string)
  has.col <- !is.null(col.string)
  has.text <- !is.null(text.string)
  has.out1 <- !is.null(out1.string)
  has.out2 <- !is.null(out2.string)
  has.dq <- !is.null(dq.string)
  if(has.num && has.text) stop("Specify either 'num' or 'text'. Not both.")

  # Split a formula like: resp~x*y|cond into a list of text strings called
  # resp, xy (vector like 'x' '*' 'y') , cond ('cond' could be a vector)
  ff <- latticeParseFormula(form, data)
  ff <- list(resp = ff$left.name,
             xy = strsplit(ff$right.name, " ")[[1]],
             cond = names(ff$condition))
  if(length(ff$resp)==0L) ff$resp <- NULL

  fill.string <- ff$resp
  x.string <- ff$xy[1]
  y.string <- ff$xy[3]
  panel.string <- ff$cond[1]

  # Resolve the (overloaded) 'ticks' argument into a show flag + per-axis breaks.
  tk <- .resolve_ticks(ticks, data[[x.string]], data[[y.string]])

  # If ticks are requested, add axis labels
  if (missing(xlab))
    xlab <- ifelse(tk$show, x.string, "")
  if (missing(ylab))
    ylab <- ifelse(tk$show, y.string, "")

  if(has.col){
    data[[col.string]] <- factor(data[[col.string]]) # In case it is numeric
  } else {
    # We still need a column for color to pass to aes( color=.data[[col.string]] )
    col.string="no_color"
    data[[col.string]] <- factor(1)
  }
  
  # Determine what fills the cells: nothing, character/factor, or numeric
  if(is.null(fill.string)) fill.type="none"
  else if (is.factor(data[[fill.string]]))
    fill.type <- "factor"
  else if (is.character(data[[fill.string]])){
    data[[fill.string]] <- as.factor(data[[fill.string]])
    fill.type <- "factor"
  } else {
    fill.type <- "num"
  }
  
  # Now get the fill values/length
  if(fill.type=="none") {
    fill.val <- rep(1, nrow(data))
    fill.n <- 1
    # Hack.  We need something to plot, call it .const
    form <- as.formula(paste(".const", form[[1]], deparse(form[[2]]), sep=""))
    data[['.const']] <- fill.val
  } else if(fill.type=="num"){
    fill.val <- data[[fill.string]]
  } else { # character/factor
    fill.val <- data[[fill.string]]
    fill.n <- nlevels(fill.val)
  }

  # Define fill colors and 'at' (if not given by the user)
  # at = # cut points for region colors
   if(fill.type=="none") {
    col.regions <- "transparent"
    at <- c(0.5,1.5)
  } else if(fill.type=="factor"){
    # If col.regions is a function, switch to default fill colors
    if(is.function(col.regions))
      col.regions <- c("#E6E6E6","#FFD9D9","#FFB2B2","#FFD7B2","#FDFFB2",
                       "#D9FFB2","#B2D6FF","#C2B2FF","#F0B2FF","#A6FFC9",
                       "#FF8C8C","#B2B2B2","#FFBD80","#BFFF80","#80BAFF",
                       "#9980FF","#E680FF","#D0D192","#59FF9C","#FFA24D",
                       "#FBFF4D","#4D9FFF","#704DFF","#DB4DFF","#808080",
                       "#9FFF40","#C9CC3D")
    # Handle named vectors for col.regions
    if(!is.null(names(col.regions))) {
      fill.levels <- levels(fill.val)
      matched_colors <- col.regions[fill.levels]
      # Check if all levels were matched
      if(any(is.na(matched_colors))) {
        missing_levels <- fill.levels[is.na(matched_colors)]
        warning("col.regions: Not all factor levels found in provided names. ",
                "Missing: ", paste(missing_levels, collapse=", "),
                ". Falling back to positional matching.")
        # Strip names so scale_fill_manual() matches positionally, not by name.
        col.regions <- rep(as.vector(col.regions), length=fill.n)
      } else {
        col.regions <- as.vector(matched_colors)
      }
    } else {
      col.regions <- rep(col.regions, length=fill.n)
    }
    at <- c((0:fill.n)+.5)
  } else if(fill.type=="num") {
    if(missing(at) && is.null(midpoint)){
        nbins <- 15
        if(is.function(col.regions)) col.regions <- col.regions(nbins)
        # Use lel = lattice:::extend.limits to move breakpoints past ends of fill.val
        zrng <- lel(range(as.numeric(fill.val), finite = TRUE))
        at <- seq(zrng[1], zrng[2], length.out = 16)
    }
    if(missing(at) && midpoint=="median"){ # default case for continuous data
      if(is.function(col.regions)) {
        nbins <- 15
        col.regions <- col.regions(nbins)
      } else {
        nbins <- length(col.regions)
      }
      med <- median(fill.val, na.rm=TRUE)
      radius <- max(max(fill.val, na.rm=TRUE)-med,
                    med-min(fill.val, na.rm=TRUE)) + .Machine$double.eps
      zrng <- lel(range(c(med-radius, med+radius)))
      brks <- seq(zrng[1], zrng[2], length.out = nbins+1)
      binno <- as.numeric(cut(fill.val, breaks=brks)) # bin number for each fill.val
      # select only 'col.regions' and 'at' values we actually need
      minbin <- min(binno, na.rm=TRUE); maxbin <- max(binno, na.rm=TRUE)
      col.regions <- col.regions[minbin:maxbin]
      at <- brks[minbin:(maxbin+1)]
    }
    if(missing(at) && midpoint=="midrange"){ # halfway between min & max
      if(is.function(col.regions)) {
        nbins <- 15
        col.regions <- col.regions(nbins)
      } else {
        nbins <- length(col.regions)
      }
      med <- median(range(fill.val, na.rm=TRUE))
      radius <- max(max(fill.val, na.rm=TRUE)-med,
                    med-min(fill.val, na.rm=TRUE)) + .Machine$double.eps
      zrng <- lel(range(c(med-radius, med+radius)))
      brks <- seq(zrng[1], zrng[2], length.out = nbins+1)
      binno <- as.numeric(cut(fill.val, breaks=brks)) # bin number for each fill.val
      # select only 'col.regions' and 'at' values we actually need
      minbin <- min(binno, na.rm=TRUE); maxbin <- max(binno, na.rm=TRUE)
      col.regions <- col.regions[minbin:maxbin]
      at <- brks[minbin:(maxbin+1)]
    }
    if(missing(at) && is.numeric(midpoint)){
      if(is.function(col.regions)) {
        nbins <- 15
        col.regions <- col.regions(nbins)
      } else {
        nbins <- length(col.regions)
      }
      radius <- max(max(fill.val, na.rm=TRUE)-midpoint,
                    midpoint-min(fill.val, na.rm=TRUE)) + .Machine$double.eps
      zrng <- lel(range(c(midpoint-radius, midpoint+radius)))
      brks <- seq(zrng[1], zrng[2], length.out = nbins+1)
      binno <- as.numeric(cut(fill.val, breaks=brks)) # bin number for each fill.val
      # select only col.regions and at we actually need
      minbin <- min(binno, na.rm=TRUE); maxbin <- max(binno, na.rm=TRUE)
      col.regions <- col.regions[minbin:maxbin]
      at <- brks[minbin:(maxbin+1)]
    }
    if(!missing(at)){
      # user specified 'at' and 'col.regions'
      nbins <- length(at)-1
      if(is.function(col.regions)) col.regions <- col.regions(nbins)
    }
    
  } # end fill.type
  # comment: the Fields package defines breakpoints so that the first and last
  # bins have their midpoints at the minimum and maximum values in z
  # https://www.image.ucar.edu/~nychka/Fields/Help/image.plot.html
  
  # Text colors
  if(is.null(col.text))
    col.text <- c("black", "red3", "darkorange2", "chartreuse4",
                  "deepskyblue4", "blue", "purple4", "darkviolet", "maroon")

  # Change x/y from factor to numeric if needed.  Add missing x,y levels.
  fac2num <- function(x) as.numeric(levels(x))[x]
  if(is.factor(data[[x.string]])) data[[x.string]] <- fac2num(data[[x.string]])
  if(is.factor(data[[y.string]])) data[[y.string]] <- fac2num(data[[y.string]])

  # Combine ALL conditioning variables into a single panel factor, so a formula
  # like 'y ~ x*z | a + b' facets on every conditioning variable. Previously only
  # ff$cond[1] was used, silently dropping the rest and overplotting cells.
  if(length(ff$cond) > 0L){
    panel.string <- ".panel"
    data[[panel.string]] <- interaction(data[ff$cond], sep=" | ",
                                        drop=TRUE, lex.order=TRUE)
  } else {
    panel.string <- NULL
  }

  # Why is .addLevels used here but not in desplot?  Because ggplot2 does not 
  # automatically add missing factor levels to the plot, so we need to do it 
  # manually.  In desplot, the lattice package automatically adds missing 
  # factor levels to the plot.
  data <- .addLevels(data, x.string, y.string, panel.string)

  # Check for multiple values
  if(is.null(panel.string)){
    tt <- table(data[[x.string]], data[[y.string]])
  } else {
    tt <- table(data[[x.string]], data[[y.string]], data[[panel.string]])
  }
  if(any(tt>1))
    warning("There are multiple data for each x/y/panel combination")

  # Calculate 'lr' rows in legend, 'lt' legend text strings
  lr <- 0
  lt <- NULL

  if(has.out1){ # out1
    lr <- lr + 1
    lt <- c(lt, out1.string)
  }
  if(has.out2){ # out2
    lr <- lr + 1
    lt <- c(lt, out2.string)
  }
  if(has.out1 | has.out2) lr <- lr + 1 # blank line

  if(fill.type=="factor") { # fill
    lt.fill <- levels(fill.val)
    lr <- lr + 2 + fill.n
    lt <- c(lt, lt.fill)
  }

  if(has.num) { # number
    num.val <- factor(data[[num.string]])
    lt.num <- levels(num.val)
    num.n <- length(lt.num)
    lr <- lr + 2 + num.n
    lt <- c(lt, lt.num)
  }

  # fixme, col.val not used anywhere
  if(has.col) { # color
    col.val <- data[[col.string]]
    lt.col <- levels(col.val)
    col.n <- length(lt.col)
    lr <- lr + 2 + col.n
    lt <- c(lt, lt.col)
    # Handle named vectors for col.text
    if(!is.null(names(col.text))) {
      matched_colors <- col.text[lt.col]
      # Check if all levels were matched
      if(any(is.na(matched_colors))) {
        missing_levels <- lt.col[is.na(matched_colors)]
        warning("col.text: Not all factor levels found in provided names. ",
                "Missing: ", paste(missing_levels, collapse=", "),
                ". Falling back to positional matching.")
        # Strip names so scale_color_manual() matches positionally, not by name.
        col.text <- rep(as.vector(col.text), length=col.n)
      } else {
        col.text <- as.vector(matched_colors)
      }
    } else {
      if(length(col.text) < col.n) col.text <- rep(col.text, length=col.n)
    }
  } else {
    col.val <- rep(1, nrow(data)) # No color specified, use black by default
  }

  if(has.text) { # text
    text.val <- factor(data[[text.string]]) # In case it is not a factor
    lt.text <- levels(text.val)
    text.n <- length(lt.text)
    lr <- lr + 2 + text.n
    lt <- c(lt, lt.text)
  }

  # Set up short version of text
  if(has.text & is.null(text.levels)){
    if(shorten=='no' | shorten=='none' | (is.logical(shorten) && !shorten))
      text.levels <- lt.text
    else if (shorten=='abb')
      text.levels <- abbreviate(lt.text, 2, method='both')
    else if (shorten=='sub')
      text.levels <- substring(lt.text, 1, 3)
  }

  # We might not have a key, even though it was requested
  if (lr==0) show.key <- FALSE

  # In function call we use 'list' instead of 'gpar' because gpar is not
  # exported from grid, so now fixup the class for out1.gpar, out2.gpar
  #if(class(out1.gpar) != "gpar") class(out1.gpar) <- "gpar"
  #if(class(out2.gpar) != "gpar") class(out2.gpar) <- "gpar"
  class(out1.gpar) <- "gpar"
  class(out2.gpar) <- "gpar"

  # Cell text
  if(has.text) {
    data$cell.text <- text.levels[as.numeric(text.val)]
  } else if(has.num) {
    data$cell.text <- as.numeric(num.val)
  } else if(has.col) {
    data$cell.text <- rep("x", length=nrow(data))
  }

  # Data quality flag
  if(has.dq) {
    data$dq.val <- factor(data[[dq.string]])
    levels(data$dq.val) <- list("0"=c("0","G","Good"),
                           "1"=c("1","Q","Questionable"),
                           "2"=c("2","B","Bad","S","Suppressed"))
    data$dq.val <- as.numeric(as.character(data$dq.val))
  }

  # --------------- build the plot ---------------

  #out <- ggplot(data, aes_string(x=x.string, y=y.string))
  out <- ggplot(data, aes(x=.data[[x.string]], y=.data[[y.string]]))
  
  if(!is.null(panel.string))
    out <- out + 
      facet_wrap(panel.string, scales="free")
  
  # Note, cells with a missing value are left empty instead of being filled
  # with the ggplot2 default grey50, which is hard to tell from the lightgray
  # midpoint of RedGrayBlue. The lattice version leaves such cells empty too.
  if(fill.type=="none")
    # No fill variable (empty formula LHS). Draw an invisible tile grid anyway
    # so the +-0.5 cell extent is established; otherwise num/text labels on the
    # outer cells sit on the panel border and are clipped. The lattice path
    # reserves this extent regardless of fill.
    out <- out +
    geom_tile(fill = "transparent")

  if(fill.type=="num")
    out <- out +
    #geom_tile(aes_string(fill = fill.string)) +
    geom_tile(aes(fill = .data[[fill.string]])) +
    scale_fill_gradientn(colours=col.regions, guide="colorbar",
                         na.value="transparent")
  
  if(fill.type=="factor")
    out <- out +
    #geom_tile(aes_string(fill = fill.string)) +
    geom_tile(aes(fill = .data[[fill.string]])) +
    scale_fill_manual(values=col.regions, na.value="transparent")
  
  if(has.out1)
    out <- out +
    #geom_tileborder(aes_string(group=1, grp=out1.string),
    geom_tileborder(aes(group=1, grp=.data[[out1.string]]),
                    lineend="round",
                    color=out1.gpar$col,
                    lwd=out1.gpar$lwd,
                    linetype=if(is.null(out1.gpar$lty)) 1 else out1.gpar$lty)

  if(has.out2)
    out <- out +
    #geom_tileborder(aes_string(group=1, grp=out2.string),
    geom_tileborder(aes(group=1, grp=.data[[out2.string]]),
                    color=out2.gpar$col,
                    lwd=out2.gpar$lwd,
                    linetype=if(is.null(out2.gpar$lty)) 1 else out2.gpar$lty)
  # use '4*cex' so that cex in lattice/ggplot2 is roughly the same size
  if(has.text|has.num|has.col) # cell text
    #out = out + geom_text(aes_string(x.string, y.string, 
    out = out + geom_text(aes(x=.data[[x.string]], y=.data[[y.string]], 
                              label=.data[["cell.text"]], color=.data[[col.string]]),
                          size=4*cex) +
    # Only show a color key when 'col' was actually supplied; otherwise the
    # internal dummy 'no_color' factor would leak in as a spurious legend.
    scale_color_manual(values=col.text, guide=if(has.col) "legend" else "none")
  
  if(has.dq) {
    # Data quality indicator

    # draw diagonal line from lower-left to upper-right
    data$x1l = data[[x.string]]-.5
    data$x1r = data[[x.string]]+.5
    data$y1l = data[[y.string]]-.5
    data$y1r = data[[y.string]]+.5
    # need to use aes_string to prevent CRAN check
    # "no visible binding for global variable"
    # Same for data[data$dq.val ,] instead of subset
    out <- out +
      geom_segment(data=data[data$dq.val >= 1L, ],
                   #aes_string(x="x1l", xend="x1r", y="y1l", yend="y1r"),
                   aes(x=.data[["x1l"]], xend=.data[["x1r"]], y=.data[["y1l"]], yend=.data[["y1r"]]),
                   color="black")
    # draw diagonal line from upper-left to lower-right
    data$y1l = data[[y.string]]+.5
    data$y1r = data[[y.string]]-.5
    out <- out +
      geom_segment(data=data[data$dq.val >= 2L, ],
                   #aes_string(x="x1l", xend="x1r", y="y1l", yend="y1r"),
                   aes(x=.data[["x1l"]], xend=.data[["x1r"]], y=.data[["y1l"]], yend=.data[["y1r"]]),
                   color="black")
  }
  
  if(!show.key)
    out <- out + theme(legend.position="none")
  
  # axis labels
  out <- out +
    ggtitle(main) +
    theme(plot.title = element_text(hjust = 0.5)) + # center title
    xlab(xlab) + 
    ylab(ylab)
  
  # Axis breaks: honour explicit/"all" breaks from 'ticks'; keep flip via reverse.
  if(!is.null(tk$x))
    out <- out + scale_x_continuous(breaks = tk$x)
  if(flip)
    out <- out + scale_y_reverse(breaks = if(is.null(tk$y)) waiver() else tk$y)
  else if(!is.null(tk$y))
    out <- out + scale_y_continuous(breaks = tk$y)

  # remove axis ticks and labels
  if(!tk$show)
    out <- out +
      theme(axis.text.x=element_blank(),
            axis.text.y=element_blank(),
            axis.ticks=element_blank())

  # Apply aspect ratio if specified
  # Both lattice and ggplot2 use aspect as height/width ratio
  if(!is.null(aspect)) {
    out <- out + coord_fixed(ratio = aspect, expand = FALSE)
  } else {
    out <- out + coord_cartesian(expand = FALSE) # no extra space between facet heatmaps
  }

  # blank theme
  out <- out +
    theme(axis.line = if(panel.border) element_line(colour = "black") # left/bottom border
                      else element_blank(),
          panel.grid.major = element_blank(),
          panel.grid.minor = element_blank(),
          panel.border = if(panel.border) element_rect(fill = NA, colour = "black") # top/right
                         else element_blank(),
          panel.background = element_blank(),
          panel.spacing = unit(0, "lines"),   # space between panels
          strip.text = element_text(size = 11 * strip.cex)
          )
  
  out
}

# Normalize the overloaded 'ticks' argument.
# Accepts one of:
#   FALSE / TRUE  - logical (backward compatible): FALSE hides the axes, TRUE
#                   shows them with the default (pretty) breaks.
#   "all"         - show axes with a break at every integer coordinate, resolved
#                   separately for each axis.
#   list(x=, y=)  - explicit per-axis control; each element is a numeric vector of
#                   breaks or "all". A missing element leaves that axis at the
#                   default (pretty) breaks.
# 'xvals'/'yvals' are the numeric coordinate values, used to resolve "all".
# Returns list(show = <logical: draw axes?>, x = <NULL|numeric>, y = <NULL|numeric>),
# where a NULL break vector means "use the default breaks for that axis".
.resolve_ticks <- function(ticks, xvals, yvals) {
  all_int <- function(v) {
    v <- v[is.finite(v)]
    if(length(v) == 0) return(numeric(0))
    seq(floor(min(v)), ceiling(max(v)), by = 1)
  }
  one <- function(spec, vals) {
    if(is.null(spec)) return(NULL)
    if(identical(spec, "all")) return(all_int(vals))
    if(is.numeric(spec)) return(spec)
    stop("'ticks' list elements must be numeric or \"all\".", call. = FALSE)
  }
  if(is.logical(ticks)) {
    if(length(ticks) != 1L || is.na(ticks))
      stop("'ticks' must be TRUE, FALSE, \"all\", or a list(x=, y=).", call. = FALSE)
    return(list(show = ticks, x = NULL, y = NULL))
  }
  if(is.character(ticks) && length(ticks) == 1L && ticks == "all")
    return(list(show = TRUE, x = all_int(xvals), y = all_int(yvals)))
  if(is.list(ticks)) {
    bad <- setdiff(names(ticks), c("x", "y"))
    if(is.null(names(ticks)) || length(bad) > 0)
      stop("'ticks' list may only have named elements 'x' and 'y'.", call. = FALSE)
    return(list(show = TRUE, x = one(ticks$x, xvals), y = one(ticks$y, yvals)))
  }
  stop("'ticks' must be TRUE, FALSE, \"all\", or a list(x=, y=).", call. = FALSE)
}

# Regression tests for ggdesplot() bug fixes
library(desplot)

# 2 sites x 2 reps, 3x3 grid; response constant within a panel, differs across.
twocond <- expand.grid(x = 1:3, row = 1:3,
                       site = c("S1", "S2"), rep = c("R1", "R2"))
twocond$site <- factor(twocond$site)
twocond$rep  <- factor(twocond$rep)
twocond$y    <- as.numeric(interaction(twocond$site, twocond$rep))

# 4 fill levels but names only cover two -> triggers positional fallback
partial <- data.frame(
  col = rep(1:4, times = 4),
  row = rep(1:4, each = 4),
  rep = factor(paste0("R", rep(1:4, length.out = 16)))
)

# 6 columns x 4 rows: x- and y-range differ, so "all" resolves per-axis and
# 'pretty' (2,4,6) is distinguishable from 'all' (1:6).
fld <- expand.grid(COLUMN = 1:6, ROW = 1:4)
fld$ENTRY <- factor(seq_len(nrow(fld)) %% 12 + 1)
xbreaks <- function(p) { b <- ggplot2::ggplot_build(p)$layout$panel_params[[1]]$x$breaks; b[!is.na(b)] }
ybreaks <- function(p) { b <- ggplot2::ggplot_build(p)$layout$panel_params[[1]]$y$breaks; b[!is.na(b)] }

test_that("ggdesplot does not add a spurious colour legend without 'col'", {
  skip_if_not(utils::packageVersion("ggplot2") >= "3.5.0")
  data(besag.met, package = "agridat")
  p <- ggdesplot(besag.met, yield ~ col * row|county, 
           text = gen)
  # the dummy 'no_color' aesthetic must not produce a drawn colour guide
  expect_null(ggplot2::get_guide_data(p, "colour"))
})

test_that("ggdesplot named col.regions fallback colours every level", {
  # the names cover only R1/R2 of four levels: a deliberate warning fires and
  # the code falls back to positional matching. Assert the warning (not suppress
  # it) and then check the fallback still colours every tile (no grey NA).
  expect_warning(
    p <- ggdesplot(partial, rep ~ col * row, col.regions = c(R1 = "red", R2 = "blue")),
    "positional"
  )
  b <- ggplot2::ggplot_build(p)
  expect_false(any(is.na(b$data[[1]]$fill)))
})

test_that("ggdesplot with no fill variable keeps num/text labels inside the panel", {
  # Empty LHS: cells carry only numbers (num=), no fill. Without a tile to
  # establish the +-0.5 cell extent, boundary labels are drawn on the panel
  # edge and get clipped (the num= plot looked wrong). A transparent tile grid,
  # as in the lattice path, must provide that extent.
  blk <- expand.grid(col = 1:2, row = 1:3, block = c("B1", "B2"))
  blk$block <- factor(blk$block)
  blk$g     <- factor(rep(1:3, length.out = nrow(blk)))
  p <- ggdesplot(blk, ~ col * row | block, num = g)
  b <- ggplot2::ggplot_build(p)
  rng <- b$layout$panel_params[[1]]$x.range
  # the drawn range must reach half a cell beyond the outer column centres,
  # otherwise a label at col 1 or col 2 sits on the panel border
  expect_lte(rng[1], 0.5)
  expect_gte(rng[2], 2.5)
  # the extent comes from a (transparent) tile layer, not from the text alone
  expect_true(any(vapply(p$layers, function(l) inherits(l$geom, "GeomTile"), logical(1))))
})

test_that("ggdesplot facets on every conditioning variable", {
  p <- ggdesplot(twocond, y ~ x * row | site + rep)
  b <- ggplot2::ggplot_build(p)
  expect_equal(length(unique(b$data[[1]]$PANEL)), 4L)
  # each panel holds exactly its 9 cells (no overplotting from a dropped factor)
  expect_true(all(table(b$data[[1]]$PANEL) == 9L))
})

test_that("ggdesplot single conditioning variable keeps its panel labels", {
  data(besag.met, package = "agridat")
  p <- ggdesplot(besag.met, yield ~ col * row | county)
  expect_equal(levels(p$data$.panel), paste0("C", 1:6))
})

test_that("ggdesplot leaves a cell with a missing value empty", {
  # 4x4 field with a hole at col 2 / row 2, once numeric and once a factor
  hole <- data.frame(
    col = rep(1:4, times = 4),
    row = rep(1:4, each = 4),
    yield = c(1:5, NA, 7:16),
    B = factor(c(rep("b1", 5), NA, rep("b2", 10)))
  )

  fill_num <- ggplot2::layer_data(ggdesplot(hole, yield ~ col * row), 1)$fill
  expect_equal(sum(fill_num == "transparent"), 1L)
  # the guard: the other 15 cells still get a real colour from col.regions,
  # so the empty cell comes from the missing value and not from a broken scale
  expect_equal(length(unique(fill_num[fill_num != "transparent"])), 15L)

  fill_fac <- ggplot2::layer_data(ggdesplot(hole, B ~ col * row), 1)$fill
  expect_equal(sum(fill_fac == "transparent"), 1L)
  expect_equal(length(unique(fill_fac[fill_fac != "transparent"])), 2L)
})

test_that("ggdesplot panel.border switch toggles the panel border and axis line", {
  p_on  <- ggdesplot(fld, ENTRY ~ COLUMN * ROW)
  p_off <- ggdesplot(fld, ENTRY ~ COLUMN * ROW, panel.border = FALSE)
  expect_true(inherits(p_on$theme$panel.border,  "element_rect"))   # default keeps it
  expect_true(inherits(p_off$theme$panel.border, "element_blank"))  # switch removes it
  expect_true(inherits(p_off$theme$axis.line,    "element_blank"))  # axis.line follows
})

test_that("ggdesplot ticks='all' puts a break at every integer, resolved per axis", {
  p <- ggdesplot(fld, ENTRY ~ COLUMN * ROW, ticks = "all")
  expect_equal(xbreaks(p), as.numeric(1:6))
  expect_equal(ybreaks(p), as.numeric(1:4))
  expect_false(inherits(p$theme$axis.text.x, "element_blank"))      # axes shown
  # flip keeps the same data-space breaks (scale_y_reverse negates the positions)
  p_flip <- ggdesplot(fld, ENTRY ~ COLUMN * ROW, ticks = "all", flip = TRUE)
  expect_true(setequal(abs(ybreaks(p_flip)), 1:4))
})

test_that("ggdesplot ticks=list gives explicit per-axis breaks", {
  p <- ggdesplot(fld, ENTRY ~ COLUMN * ROW, ticks = list(x = c(1, 3, 5), y = "all"))
  expect_equal(xbreaks(p), c(1, 3, 5))
  expect_equal(ybreaks(p), as.numeric(1:4))
  # a missing list element leaves that axis at the default (pretty) breaks, axes still shown
  p2 <- ggdesplot(fld, ENTRY ~ COLUMN * ROW, ticks = list(x = c(2, 4)))
  expect_equal(xbreaks(p2), c(2, 4))
  expect_equal(ybreaks(p2), ybreaks(ggdesplot(fld, ENTRY ~ COLUMN * ROW, ticks = TRUE)))
  expect_false(inherits(p2$theme$axis.text.x, "element_blank"))
})

test_that("ggdesplot logical ticks stay backward compatible", {
  p_f <- ggdesplot(fld, ENTRY ~ COLUMN * ROW, ticks = FALSE)
  p_t <- ggdesplot(fld, ENTRY ~ COLUMN * ROW, ticks = TRUE)
  expect_true(inherits(p_f$theme$axis.text.x,  "element_blank"))    # FALSE hides axes
  expect_false(inherits(p_t$theme$axis.text.x, "element_blank"))    # TRUE shows them
  expect_false(identical(xbreaks(p_t), as.numeric(1:6)))            # pretty (2,4,6), not 'all'
})

test_that("ggdesplot rejects an invalid ticks specification", {
  expect_error(ggdesplot(fld, ENTRY ~ COLUMN * ROW, ticks = "foo"))
  expect_error(ggdesplot(fld, ENTRY ~ COLUMN * ROW, ticks = list(z = 1)))
  expect_error(ggdesplot(fld, ENTRY ~ COLUMN * ROW, ticks = NA))
})

# ---------------------------------------------------------------------------
# Same 'ticks' / 'panel.border' features on the lattice path, so desplot() and
# ggdesplot() behave identically. The lattice trellis object stores the axis
# spec in p$x.scales$draw / $at and the panel box colour in
# p$par.settings$axis.line$col.
# ---------------------------------------------------------------------------

test_that("desplot ticks='all' puts a break at every integer, resolved per axis", {
  p <- desplot(fld, ENTRY ~ COLUMN * ROW, ticks = "all")
  expect_true(isTRUE(p$x.scales$draw))
  expect_equal(as.numeric(p$x.scales$at), as.numeric(1:6))
  expect_equal(as.numeric(p$y.scales$at), as.numeric(1:4))
  # flip keeps the same break values and does not error while building
  p_flip <- desplot(fld, ENTRY ~ COLUMN * ROW, ticks = "all", flip = TRUE)
  expect_equal(as.numeric(p_flip$y.scales$at), as.numeric(1:4))
})

test_that("desplot ticks=list gives explicit per-axis breaks", {
  p <- desplot(fld, ENTRY ~ COLUMN * ROW, ticks = list(x = c(1, 3, 5), y = "all"))
  expect_equal(as.numeric(p$x.scales$at), c(1, 3, 5))
  expect_equal(as.numeric(p$y.scales$at), as.numeric(1:4))
  # a missing list element leaves that axis at lattice's default (at = FALSE)
  p2 <- desplot(fld, ENTRY ~ COLUMN * ROW, ticks = list(x = c(2, 4)))
  expect_equal(as.numeric(p2$x.scales$at), c(2, 4))
  expect_identical(p2$y.scales$at, FALSE)
  expect_true(isTRUE(p2$x.scales$draw))
})

test_that("desplot logical ticks stay backward compatible", {
  p_f <- desplot(fld, ENTRY ~ COLUMN * ROW, ticks = FALSE)
  p_t <- desplot(fld, ENTRY ~ COLUMN * ROW, ticks = TRUE)
  expect_true(isFALSE(p_f$x.scales$draw))                         # FALSE hides axes
  expect_true(isTRUE(p_t$x.scales$draw))                          # TRUE shows them
  expect_false(identical(as.numeric(p_t$x.scales$at), as.numeric(1:6)))  # default breaks, not 'all'
})

test_that("desplot panel.border switch toggles the panel box and axis line", {
  desplot(besag.met, yield ~ col*row|county, panel.border=FALSE)
  p_on  <- desplot(fld, ENTRY ~ COLUMN * ROW)
  p_off <- desplot(fld, ENTRY ~ COLUMN * ROW, panel.border = FALSE)
  expect_false(identical(p_on$par.settings$axis.line$col, "transparent"))  # default keeps it
  expect_identical(p_off$par.settings$axis.line$col, "transparent")        # switch removes it
  # a par.settings the user passes through '...' survives the merge
  p_mix <- desplot(fld, ENTRY ~ COLUMN * ROW, panel.border = FALSE,
                   par.settings = list(strip.background = list(col = "grey90")))
  expect_identical(p_mix$par.settings$axis.line$col, "transparent")
  expect_identical(p_mix$par.settings$strip.background$col, "grey90")
})

test_that("desplot rejects an invalid ticks specification", {
  expect_error(desplot(fld, ENTRY ~ COLUMN * ROW, ticks = "foo"))
  expect_error(desplot(fld, ENTRY ~ COLUMN * ROW, ticks = list(z = 1)))
  expect_error(desplot(fld, ENTRY ~ COLUMN * ROW, ticks = NA))
})

require(desplot)

# ----------------------------------------------------------------------------
# create data for tests

dat0 <- data.frame(loc = c('loc1','loc1','loc1','loc1',
                           'loc2','loc2','loc2','loc2','loc2','loc2'),
                   x=c(1,2,1,2, 1,2,3,1,2,3),
                   y=c(1,1,2,2, 1,1,1,2,2,2),
                   rep=c('R1','R1','R2','R2',' R1','R2','R3','R1','R2','R3'),
                   yield=c(9.29,11.20,9.36,9.89,8.47,9.17,8.86,10.48,10.22,11.29),
                   trt1=c('Treat1','Treat2','Treat2','Treat1',
                          'Trt1','Trt2','Trt1','Trt2','Trt1','Trt2'),
                   trt2=c('Hybrid1','Hybrid1','Hybrid2','Hybrid2','Hybrid1',
                          'Hybrid2','Hybrid3','Hybrid1','Hybrid2','Hybrid3'),
                   trt3=c("A","A","B","B","A","A","A","A","A","A"),
                   dq=c(0,0,0,1,2))
dat3 <- data.frame(
  yield = 7:1,
  x = c(5,  6,     1,  3,  4, 5, 5),
  y = c(2,  3,     1,  2,  3, 1, 2),
  loc = c("L1", "L1",   "L2", "L2", "L2", "L2", "L2"),
  block = c("B1","B1", "B1","B1","B2","B2","B2"))

require(agridat)
data(yates.oats, package="agridat")
oats35 <- yates.oats
oats35$dq <- rep(c(0,0,0,0,0,0,0,0,1,2), length=72)
# ----------------------------------------------------------------------------

# aspect ratio
#data(yates.oats)

# Show experiment layout in true aspect
# Field width = 4 plots * 44 links = 176 links
# Field length = 18 plots * 28.4 links = 511 links
test_that("aspect ratio", {
  # With lattice, the aspect ratio is y/x for the entire field
  expect_silent(
    desplot(yates.oats, yield ~ col+row, aspect=511/176)
  )
  # With ggplot, the aspect ratio is y/x for each cell
  expect_silent(
    ggdesplot(yates.oats, yield ~ col+row, aspect=28.4/44)
  )
  })


## ---------------------------------------------------------------------------

# col.regions, at, midpoint

test_that("num,col,text", {
  expect_silent({
    desplot(oats35, ~col+row|block, cex=1, num=gen)
    desplot(oats35, ~col+row|block, cex=1, col=gen)
    desplot(oats35, ~col+row|block, cex=1, text=gen)
    desplot(oats35, ~col+row|block, cex=1, num=gen,col=nitro)
    desplot(oats35, ~col+row|block, cex=1, text=gen,col=nitro)
  })
})

test_that("function cleanup checks variables", {
  expect_error( desplot(oats35, yield~col+row, num=junk) )
  expect_error( desplot(oats35, yield~col+row, col=junk) )
  expect_error( desplot(oats35, yield~col+row, text=junk) )
  expect_error( desplot(oats35, yield~col+row, out1=junk) )
  expect_error( desplot(oats35, yield~col+row, out2=junk) )
})


test_that("shorten", {
  expect_silent({
    desplot(oats35, block~col+row, text=gen, cex=1) # default shorten='abb'
    desplot(oats35, block~col+row, text=gen, cex=1, shorten='abb')
    desplot(oats35, block~col+row, text=gen, cex=1, shorten='sub')
    desplot(oats35, block~col+row, text=gen, cex=1, shorten='no')
    desplot(oats35, block~col+row, text=gen, cex=1, shorten='none')
    desplot(oats35, block~col+row, text=gen, cex=1, shorten=FALSE)
  })
})

test_that("out1,out2,out1.gpar,out2.gpar", {
  expect_silent({
    desplot(oats35, yield~col+row, out1=block)
    desplot(oats35, yield~col+row, out2=block)
    desplot(oats35, yield~col+row, out1=block, out2=gen)
    desplot(oats35, yield~col+row,
            out1=block, out1.gpar=list(col="white",lwd=2))
    desplot(oats35, yield~col+row,
            out2=block, out2.gpar=list(col="deeppink",lwd=2))
    desplot(dat3, yield ~ x*y|loc, out1=block, out2=block) # no outline available
    # note, the following line gives a warning, I think because there are
    # no combinations of L1 & B2 ???
    # desplot(dat3, yield ~ x*y|loc, out1=block, gg=TRUE)
  })
})

test_that("data quality argument works", {
  expect_silent({
    desplot(oats35, block~col+row, out1=block, dq=dq)
    desplot(oats35, block~col+row|block, out1=block, dq=dq)
  })
})

test_that("strip.cex", {
  expect_silent({
    desplot(dat0, yield ~ x+y|loc, strip.cex=1.5)
  })
})

test_that("text.levels", {
  expect_silent({
    desplot(oats35, block~col+row, col="nitro",
            text="gen", cex=1, text.levels=c('V','G','M'))
  })
})

test_that("key.cex, show.key", {
  expect_silent({
    desplot( dat0, ~ x+y|loc,
             text=trt1, key.cex=1, shorten='none')
    desplot( dat0, ~ x+y|loc,
             text=trt1, cex=.8, shorten='none', show.key=FALSE)
  })
})

test_that("text,xlab,ylab", {
  expect_silent({
    desplot(dat0, rep ~ x+y|loc,
            main="title", xlab="xlab", ylab="ylab")
  })
})

test_that("tick,flip", {
  expect_silent({
    desplot(oats35, yield~col+row, tick=TRUE)
    desplot(oats35, yield~col+row, tick=TRUE, flip=TRUE)  
  })
})

test_that("other arguments via ...", {
  expect_silent({
    desplot(dat0, rep ~ x+y|loc)
    desplot(dat0, rep ~ x+y|loc, aspect=1)
  })
})

test_that("subset", {
  expect_silent({
    desplot(oats35, ~col+row|block,
            subset = block %in% c("B1","B2"), cex=1, num=gen)
  })
})

## ---------------------------------------------------------------------------

test_that("If a cell has multiple observations, issue a warning.",{
  # No panel factor
  d0 <- expand.grid(col=1:2, row=1:2)
  d0$y <- rnorm(nrow(d0))
  d0 <- rbind(d0, d0[1,]) # Create one cell that has 2 observations

  # One panel factor
  d1 <- expand.grid(state=c("S1","S2"), col=1:2, row=1:2)
  d1$y <- rnorm(nrow(d1))
  d1 <- rbind(d1, d1[1,]) # Create one cell that has 2 observations

  # Two panel factors
  d2 <- expand.grid(state=c("S1","S2"), loc=c("L1","L2","L3"), col=1:2, row=1:2)
  d2$y <- rnorm(nrow(d2))
  d2 <- rbind(d2, d2[1,]) # Create one cell that has 2 observations
  
  expect_warning( desplot(d0, y~col*row) )
  expect_warning( desplot(d1, y~col*row|state) )
  expect_warning( desplot(d2, y~col*row|state*loc) )
})


# ----------------------------------------------------------------------------

desplot(dat0, ~ x+y|loc,
        text=trt2, col=trt1, cex=1,
        col.text=c('red','black','blue','plum'),
        text.levels=c('A','B','C'))




oats34 <- oats35
# create a row of missing values in the field to test .addLevels
oats34[which(oats34$row==16),'yield'] <- NA
desplot(oats34, yield~col+row, tick=TRUE)
desplot(oats34, yield~col+row|block, tick=TRUE)

# Text over continuous colors
desplot(oats35, yield~col+row,
        out1=block, text=gen, cex=1,
        xlab="x axis", ylab="y axis", ticks=TRUE)

# Test 'at' and 'col.regions' for the ribbon
RedYellowBlue <-
  colorRampPalette(c("#D73027", "#F46D43", "#FDAE61", "#FEE090", "#FFFFBF",
                     "#E0F3F8", "#ABD9E9", "#74ADD1", "#4575B4"))
eightnum <- function(x) {
  x <- x[!is.na(x)]
  st <- grDevices::boxplot.stats(x)$stats
  # eps <- .Machine$double.eps
  eps <- 10^(log10(127.4)-6)
  c(min(x)-eps, st[1:2], st[3]-eps, st[3]+eps, st[4:5], max(x)+eps)
}

desplot(oats35, yield~col+row, col.regions=RedYellowBlue(7))
# extreme values barely visible on ribbon
desplot(oats35, yield~col+row, at=eightnum(oats35$yield), midpoint=NULL)

# fails
#desplot(oats35, yield~col+row, col.regions=RedYellowBlue(7),
#        at=eightnum(oats35$yield))

# Midpoint options
# mean=103.97
# median=102.5
# middle of range =113.5
desplot(oats35, yield~col+row) # default is median
desplot(oats35, yield~col+row, midpoint="median") # same as default
desplot(oats35, yield~col+row, midpoint=102.5) # same as default
desplot(oats35, yield~col+row, midpoint="midrange")
desplot(oats35, yield~col+row, midpoint=NULL) # same as midrange
desplot(oats35, yield~col+row, midpoint=113.5) # same as midrange
desplot(oats35, yield~col+row, midpoint=103.97) # custom, mean
desplot(oats35, yield~col+row, midpoint=0) # nonsensical low
desplot(oats35, yield~col+row, midpoint=200) # nonsensical hi

# Manually remove the ribbon completely
dd <- desplot(oats35, yield ~  col+row)
dd
dd$legend$right=NULL
dd

# What if the response is character?  Treat it like a factor
oats35$genchar <- as.character(oats35$gen)
desplot(oats35, gen~col+row, col=block, num=nitro, cex=1, out1=block)
desplot(oats35, genchar~col+row, col=block, num=nitro, cex=1, out1=block)

# Show actual yield values
desplot(oats35, block~col+row, text=yield, shorten='no')

desplot(oats35, block~col+row, col=nitro, text=gen, cex=1, out1=block)
desplot(oats35, block~col+row, col=nitro, text=gen, cex=1, out1=block, out2=gen)
desplot(oats35, block~col+row, num="gen", col="nitro", cex=1)

desplot(oats35, nitro~col+row, text="gen", cex=.9)
desplot(oats35, nitro~col+row)
desplot(oats35, nitro~col+row|block, text="gen", cex=.9)


desplot(oats35, nitro~col+row|block, text="gen", cex=1)
desplot(oats35, block~col+row|block, col="nitro", text="gen", cex=1)

# Test cases with 1 or 2 rows or columns

dmet <- agridat::besag.met

desplot(dmet, yield~col*row|county, out1=rep, out2=block, tick=TRUE)

# Create new data in which C1 has one row, C2 two rows
# C4 has one column, C5 two columns
dmet2 <- dmet
dmet2 <- subset(dmet2, !(county=="C1" & row<18))
dmet2 <- subset(dmet2, !(county=="C2" & row<17))
dmet2 <- subset(dmet2, !(county=="C4" & col<11))
dmet2 <- subset(dmet2, !(county=="C5" & col<10))

desplot(dmet2, yield~col*row|county, tick=TRUE)
desplot(dmet2, yield~col*row|county, out1=rep, out2=block, tick=TRUE)

# Another midpoint example with strong difference between midpoint styles.
if(FALSE){
  library(agridat)
  # The high outlier makes most of the field look red
  desplot(wiedemann.safflower.uniformity, yield~col*row,
          flip=TRUE, tick=TRUE, aspect =99/165, # true aspect
          main="wiedemann.safflower.uniformity - midpoint='midrange'",
          midpoint=NULL) # same as "midrange"
  # Using the median balances the blue/red colors better
  desplot(wiedemann.safflower.uniformity, yield~col*row,
          flip=TRUE, tick=TRUE, aspect =99/165, # true aspect
          main="wiedemann.safflower.uniformity - midpoint='median'",
          midpoint="median")
  desplot(wiedemann.safflower.uniformity, yield~col*row,
          flip=TRUE, tick=TRUE, aspect =99/165, # true aspect
          main="wiedemann.safflower.uniformity - eightnum",
          at=eightnum(wiedemann.safflower.uniformity$yield))

}

# Test for issue #9. First panel has only 1 row.
dat1row <- data.frame(
  var = letters[1:9],
  block = as.factor(c(1, 1, 1, 2, 2, 2,2,2,2)),
  row = c(rep(1L,6),2,2,2),
  col = c(1:6,4,5,6)
)
desplot(data = dat1row,
        tick=TRUE,
        form = var ~ col + row | block)

# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------



# ggdesplot
test_that("ggdesplot", {
  data(yates.oats, package="agridat")
  oats35 <- yates.oats
  expect_silent({
    desplot(oats35, ~ col+row|block, cex=1, num=gen)
    ggdesplot(oats35, ~ col+row|block, cex=1, num=gen)
  })
} )

# geom_tileborder.R

if(0){
  ggplot(agridat::besag.met, aes(x=col, y=row)) +
    facet_wrap( ~ county) +
    geom_tile(aes(fill=yield)) +
    geom_tileborder(aes(group=1, grp=rep), lwd=1.5, lineend="round") +
    geom_tileborder(aes(group=1, grp=block), color="yellow", lwd=0.5)
  
    desplot(yield~col*row|county, agridat::besag.met, out1=rep, out2=block)
    ggdesplot(yield~col*row|county, agridat::besag.met, out1=rep, out2=block)
}

#' Borders between tiles
#' 
#' `geom_tileborder` draws a border between tiles of different classes.
#' The required aesthetics are `aes(x,y,grp)`, where `grp` is the grouping
#' classification that separates tiles.
#' 
#' Note, we cannot use `aes(group)` because it groups the interaction of
#' ALL discrete variables including facets.  Since we do not want to draw
#' a border between facets, we had to define a new aesthetic. 
#' See: # http://ggplot2.tidyverse.org/reference/aes_group_order.html
#' 
#' Also, we do not want to split the data into separate groups for each level 
#' of `grp`, so we need to include `aes(group=1)`.
#' 
#' @inheritParams ggplot2::layer
#' @inheritParams ggplot2::geom_segment
#' @import ggplot2
#' @export
#' 
#' @examples
#' dd <- data.frame(
#'   x=c(1,2,1,2,3,1,2,1,2,3),
#'   y=c(2,2,2,2,2,1,1,1,1,1),
#'   loc=factor(c(1,1,2,2,2,1,1,2,2,2)),
#'   rep=factor(c(2,2,1,2,3,1,1,1,2,3)))
#' library(ggplot2)
#' ggplot(dd, aes(x=x, y=y)) +
#'   facet_wrap( ~ loc) +
#'   geom_tile(aes(fill=rep)) +
#'   geom_tileborder(aes(group=1, grp=rep), lwd=1.5)
#' # Compare to lattice version of desplot
#' # desplot::desplot(rep ~ x*y|loc, data=dd, out1=rep)
#' 
geom_tileborder<- function(mapping = NULL, data = NULL, geom = "segment",
                           position = "identity", na.rm = TRUE, 
                           show.legend = NA, 
                           inherit.aes = TRUE, ...) {
  ggplot2::layer(
    geom = geom,
    stat = StatTileBorder, 
    data = data, 
    mapping = mapping,  
    position = position, 
    show.legend = show.legend, 
    inherit.aes = inherit.aes,
    params = list(na.rm = na.rm, ...)
  )
}


#' @rdname geom_tileborder
#' @format NULL
#' @usage NULL
#' @importFrom ggplot2 ggproto Stat
#' @export
StatTileBorder <-
  ggplot2::ggproto("StatTileBorder",
          ggplot2::Stat,
          required_aes=c("x","y","grp"),
          compute_group = function(data, scales) {
            # print(data) # so we can see the data groups
            #cat("hello\n")
            calc_borders(data$x, data$y, data$grp)
          })

#' @noRd
#' @importFrom reshape2 acast melt
calc_borders <- function(x,y,grp){
  # x,y: coordinates for tiles in a heatmap
  # grp: class identifier for each tile
  # Create a data frame with all line segments needed to draw borders between 
  # tiles in different grp. 
  # Note, NA values in grp are not considered a separate class, because we
  # do NOT want to outline missing values in the heatmap.

  # Create a matrix with the class membership in each cell.
  # Note, the 'mat' matrix is upside down from the levelplot
  dat <- data.frame(x=x, y=y, grp=grp)
  mat <- reshape2::acast(dat, y ~ x, value.var="grp")
  
  # Since 'mat' could be 1 row or column, surround it with NAs
  mat <- cbind(NA, rbind(NA, mat, NA), NA)
  
  # Find cells with top borders
  top <- mat[2:nrow(mat)-1, ] != mat[2:nrow(mat), ]
  top <- reshape2::melt(top)
  top <- top[!(top$value==FALSE | is.na(top$value)),]
  names(top) <- c('y','x','grp')
  # line segments above each cell
  top <- transform(top, x=x-.5, xend=x+.5, y=y+.5, yend=y+.5)
  
  # Find cells with right borders
  right <- mat[ , 2:ncol(mat)-1] != mat[ , 2:ncol(mat)]
  right <- reshape2::melt(right)
  right <- right[!(right$value==FALSE | is.na(right$value)),]
  names(right) <- c('y','x','grp')
  right <- transform(right, x=x+.5, xend=x+.5, y=y-.5, yend=y+.5)
  
  segs <- rbind(top,right)
  segs[,c("x","y","xend","yend")]
  if(nrow(segs)>0) segs else NULL
}

# desplot.R

# ----------------------------------------------------------------------------

#' Function to create a Red-Gray-Blue palette
#'
#' A function to create a Red-Gray-Blue palette.
#'
#' Using gray instead of white allows missing values to appear as white
#' (actually, transparent).
#'
#' @param n Number of colors to create
#' @return A vector of n colors.
#' @author Kevin Wright
#'
#' @examples
#' pie(rep(1,11), col=RedGrayBlue(11))
#' title("RedGrayBlue(11)")
#' @export
RedGrayBlue <- colorRampPalette(c("firebrick", "lightgray", "#375997"))

# ----------------------------------------------------------------------------

#' Plot the layout/data of a field experiment.
#'
#' Use this function to plot the layout of a rectangular lattice
#' field experiment and also the observed data values.
#' 
#' To create the plot using lattice graphics: 
#' 1. \code{desplot(...)}.
#' 
#' To create the plot using ggplot2 graphics, use one of the following:
#' 1. \code{ggdesplot(...)}.
#' 2. \code{desplot(..., gg=TRUE)}. 
#' 3. \code{options(desplot.gg=TRUE); desplot(...)}. 
#' Method 3 is useful to modify all results from existing scripts.
#' 
#' The lattice version is complete, mature, and robust.
#' The ggplot2 version is incomplete.  The legend can only show colors, 
#' and some function arguments are ignored.
#' In general, lattice graphics are about 4-5 times faster than ggplot2 graphics.

#' Not all lattice parameters are passed down to \code{xyplot}, but it
#' is possible to make almost any change to the plot by assigning the
#' desplot object to a variable and then edit the object by hand or use
#' \code{update} to modify the object.  Then print it manually.  See the
#' first example below.
#'
#' Use \code{col.regions} to specify fill colors.  This can either be a vector
#' of colors or a function that produces a vector of colors.  If the response
#' variable is a factor and \code{col.regions} is a \emph{function}, it will
#' be ignored and the cells are filled with default light-colored backgrounds
#' and a key is placed on the left.  If the response variable is
#' \emph{numeric}, the cells are colored according to \code{col.regions}, and
#' a ribbon key is placed on the right.
#' 
#' Use \code{shorten='abb'} (this is default) to shorten the cell text to 2
#' characters using the \code{abbreviate} function
#' Use \code{shorten='sub'} to use a 3-character substring.
#' Use \code{shorten='no'} or \code{shorten=FALSE} for no shortening.
#' 
#' Note that two sub-plots with identical levels of the split-plot factor can
#' be adjacent to each other by virtue of appearing in different whole-plots.
#' To correctly outline the split-plot factor, simply concatenate the
#' whole-plot factor and sub-plot factor together.
#'
#' To call this function inside another function, you can hack like this:
#' vr <- "yield"; vx <- "x"; vy <- "y";
#' eval(parse(text=paste("desplot(", vr, "~", vx, "*", vy, ", data=yates.oats)")))
#' 
#' @param data A data frame.
#' 
#' @param form A formula like \code{yield~x*y|location}. Note x,y are numeric.
#'
#' @param num Bare name (no quotes) of the column of the data to use 
#' as a factor for number-coding the text in each cell.
#' 
#' @param num.string String name of the column of the data to use 
#' as a factor for number-coding the text in each cell.
#' 
#' @param col Bare name (no quotes) of the column of the data to use
#' for color-coding the text shown in each cell.
#' 
#' @param col.string String name of the column of the data to use
#' for color-coding the text shown in each cell.
#' 
#' @param text Bare name (no quotes) of the column of the data to use 
#' for the actual text shown in each cell.
#' 
#' @param text.string String name of the column of the data to use 
#' for the actual text shown in each cell.
#' 
#' @param out1 Bare name (no quotes) of the column of the data to use 
#' for first-level outlining around blocks of cells.
#' 
#' @param out1.string String name of the column of the data to use 
#' for first-level outlining around blocks of cells.
#' 
#' @param out2 Bare name (no quotes) of the column of the data to use 
#' for second-level outlining around blocks of cells.
#' 
#' @param out2.string String name of the column of the data to use 
#' for second-level outlining around blocks of cells.
#' 
#' @param dq Bare name (no quotes) of the column of the data to use 
#' for indicating bad data quality with diagonal lines.
#' This can either be a numeric vector or a factor/text. 
#' Cells with 1/"Q"/"Questionable" have one diagonal line.
#' Cells with 2/"B"/"Bad","S","Suppressed" have crossed diagonal lines.
#' 
#' @param dq.string String name of the column of the data to use 
#' for indicating bad data quality with diagonal lines.
#' 
#' @param col.regions Colors for the fill color of cells.
#' 
#' @param col.text Vector of colors for text strings.
#' 
#' @param text.levels Character strings to use instead of default 'levels'.
#' 
#' @param out1.gpar A list of graphics parameters for first-level outlining.  
#' Can either be an ordinary \code{list()} or a call to \code{gpar()} from the
#'   \code{grid} package.
#' 
#' @param out2.gpar Graphics parameters for second-level of outlining.
#' 
#' @param at Breakpoints for the color ribbon.  Use this instead of 'zlim'.
#' Note: using 'at' causes 'midpoint' to be set to NULL.
#'
#' @param midpoint Method to find midpoint of the color ribbon.
#' One of 'midrange', 'median (default), or a numeric value.
#' 
#' @param ticks Controls the axis ticks and labels. One of: \code{FALSE}
#' (default, no axes), \code{TRUE} (axes with the default "pretty" breaks),
#' \code{"all"} (a break at every integer coordinate, resolved separately for
#' each axis), or a list \code{list(x=, y=)} for explicit per-axis control where
#' each element is a numeric vector of breaks or \code{"all"}. A missing list
#' element leaves that axis at the default breaks.
#'
#' @param panel.border If TRUE (default), draw the panel border and axis lines.
#' If FALSE, omit them for a cleaner field map.
#'
#' @param flip If TRUE, vertically flip the image.
#' 
#' @param main Main title.
#' 
#' @param xlab Label for x axis.
#' 
#' @param ylab Label for y axis.
#' 
#' @param shorten Method for shortening text in the key, either 'abb', 'sub', 'no', or FALSE.
#' 
#' @param show.key If TRUE, show the key on the left side. (This is not the ribbon.)
#' 
#' @param key.cex Left legend cex.
#'
#' @param cex Expansion factor for text/number in each cell.
#' 
#' @param strip.cex Strip cex.
#'
#' @param aspect Aspect ratio.
#' To get a map of a field with a true aspect ratio include 'aspect=ylen/xlen'.
#' For lattice, 'ylen' is the vertical length of the field and 'xlen'
#' is the horizontal length of the field.
#' For ggplot2, 'ylen' is the vertical length of each cell/plot and 'xlen'
#' is the horizontal length of each plot.
#' 
#' @param subset An expression that evaluates to logical index vector for subsetting the data.
#' 
#' @param gg If TRUE, desplot() switches to ggdesplot().
#' 
#' @param ... Other.
#' 
#' @return A lattice or ggplot2 object
#' 
#' @author Kevin Wright
#' 
#' @references
#' 
#' K. Ryder (1981).
#' Field plans: why the biometrician finds them useful.
#' \emph{Experimental Agriculture}, 17, 243--256.
#' 
#' @import grid
#' @import lattice
#' @importFrom reshape2 acast melt
#' @importFrom stats as.formula formula median update
#' @export
#' @rdname desplot
#' 
#' @examples
#' if(require(agridat)){
#' 
#' # Show how to customize any feature.  Here: make the strips bigger.
#' data(besag.met)
#' d1 <- desplot(besag.met, 
#'               yield ~ col*row|county, 
#'               main="besag.met",
#'               out1=rep, out2=block, out2.gpar=list(col="white"), strip.cex=2)
#' d1 <- update(d1, par.settings = list(layout.heights=list(strip=2)))
#' print(d1)
#' 
#' data(yates.oats)
#' # Show experiment layout in true aspect
#' # Field width = 4 plots * 44 links = 176 links
#' # Field length = 18 plots * 28.4 links = 511 links
#' # With lattice, the aspect ratio is y/x for the entire field
#' desplot(yates.oats, 
#'         yield ~ col+row, 
#'         out1=block, out2=gen, aspect=511/176)
#' # With ggplot, the aspect ratio is y/x for each cell
#' ggdesplot(yates.oats, 
#'         yield ~ col+row, 
#'         out1=block, out2=gen, aspect=28.4/44)
#' 
#' desplot(yates.oats,
#'         block ~ col+row,
#'         col=nitro, text=gen, cex=1, out1=block,
#'         out2=gen, out2.gpar=list(col = "gray50", lwd = 1, lty = 1))
#'
#' # Overloaded 'ticks' (a break at every integer) and the 'panel.border' switch
#' desplot(yates.oats, yield ~ col+row, ticks="all", panel.border=FALSE)
#'
#' }
desplot <- function(data,
                    form=formula(NULL ~ x + y),
                    num=NULL, num.string=NULL,
                    col=NULL, col.string=NULL,
                    text=NULL, text.string=NULL,
                    out1=NULL, out1.string=NULL,
                    out2=NULL, out2.string=NULL,
                    dq=NULL, dq.string=NULL,
                    col.regions=RedGrayBlue, col.text=NULL, text.levels=NULL,
                    out1.gpar=list(col="black", lwd=3),
                    out2.gpar=list(col="yellow", lwd=1, lty=1),
                    at, midpoint="median",
                    ticks=FALSE, panel.border=TRUE, flip=FALSE,
                    main=NULL, xlab, ylab,
                    shorten='abb',
                    show.key=TRUE,
                    key.cex, # left legend cex
                    cex=.4, # cell cex
                    strip.cex=.75,
                    aspect=NULL, 
                    subset=TRUE, gg=FALSE, ...){

  # subset, based on subset() function
  ix <- if (missing(subset)) 
    rep_len(TRUE, nrow(data))
  else {
    e <- substitute(subset)
    ix <- eval(e, data, parent.frame())
    if (!is.logical(ix)) 
      stop("'subset' must be logical")
    ix & !is.na(ix)
  }
  data <- data[ix, ]
  data <- droplevels(data) # In case the user called with subset(obj, ...)
  
  # Using 'at' overrides 'midpoint'
  if(!missing(at) && !is.null(midpoint))
    midpoint <- NULL

  if(!missing(at) && is.vector(col.regions) &&
       ( length(at) !=  length(col.regions)+1 ) )
    stop("Length of 'at' must be 1 more than length of 'col.regions'\n")
  
  # Assume num.string contains the name/string of a column in data. 
  # If num.string is NULL, then get its value by converting 'num' 
  # from a bare name to a string.  We MUST do this here
  # so that if we switch from desplot to ggdesplot, we can pass 
  # arguments as strings.  
  mc <- as.list(match.call())
  
  if(is.null(num.string)){
    if("num" %in% names(mc)) { # user did supply argument
      if(!is.character(mc$num) & !is.null(mc$num)) { # it is a bare name
        num.string <- deparse(substitute(num)) # evaluate to string
      }
    }
  }
  
  if(is.null(col.string)){
    if("col" %in% names(mc)) {
      if(!is.character(mc$col) & !is.null(mc$col)) {
        col.string <- deparse(substitute(col))
      }
    }
  }
  
  if(is.null(text.string)){
    if("text" %in% names(mc)) {
      if(!is.character(mc$text) & !is.null(mc$text)) {
        text.string <- deparse(substitute(text))
      }
    }
  }

  if(is.null(out1.string)){
    if("out1" %in% names(mc)) {
      if(!is.character(mc$out1) & !is.null(mc$out1)) {
        out1.string <- deparse(substitute(out1))
      }
    }
  }
  
  if(is.null(out2.string)){
    if("out2" %in% names(mc)) {
      if(!is.character(mc$out2) & !is.null(mc$out2)) {
        out2.string <- deparse(substitute(out2))
      }
    }
  }

  if(is.null(dq.string)){
    if("dq" %in% names(mc)) {
      if(!is.character(mc$dq) & !is.null(mc$dq)) {
        dq.string <- deparse(substitute(dq))
      }
    }
  }

  if(gg | isTRUE(options()$desplot.gg)) {
    #if (!requireNamespace("ggplot2"))
    #  stop("You must first install the ggplot2 package: install.packages('ggplot2')")
    out <- ggdesplot(form=form, data=data,
                     num.string=num.string, col.string=col.string,
                     text.string=text.string,
                     out1.string=out1.string, out2.string=out2.string,
                     dq.string=dq.string,
                     col.regions=col.regions, col.text=col.text,
                     out1.gpar=out1.gpar, out2.gpar=out2.gpar,
                     at=at, midpoint=midpoint,
                     ticks=ticks, panel.border=panel.border,
                     flip=flip, main=main, xlab=xlab, ylab=ylab,
                     shorten=shorten, show.key=show.key,
                     key.cex=key.cex, cex=cex, strip.cex=strip.cex,
                     subset=subset, ...)
    return(out)
  }
  
  dn <- names(data)
  checkvars <- function(x, dn){
    if(!is.null(x) && !is.element(x, dn))
      stop("Could not find '", x,"' in the data frame.")
  }
  checkvars(num.string, dn)
  checkvars(col.string, dn)
  checkvars(text.string, dn)
  checkvars(out1.string, dn)
  checkvars(out2.string, dn)
  checkvars(dq.string, dn)

  has.num <- !is.null(num.string)
  has.col <- !is.null(col.string)
  has.text <- !is.null(text.string)
  has.out1 <- !is.null(out1.string)
  has.out2 <- !is.null(out2.string)
  has.dq <- !is.null(dq.string)
  if(has.num && has.text) stop("Specify either 'num' or 'text'. Not both.")


  # Split a formula like: resp~x*y|cond into a list of text strings called
  # resp, xy (vector like 'x' '*' 'y') , cond ('cond' could be a vector)
  ff <- latticeParseFormula(form, data)
  ff <- list(resp = ff$left.name,
             xy = strsplit(ff$right.name, " ")[[1]],
             cond = names(ff$condition))
  if(length(ff$resp)==0L) ff$resp <- NULL

  fill.string <- ff$resp
  x.string <- ff$xy[1]
  y.string <- ff$xy[3]
  panel.string <- ff$cond[1]

  # Determine what fills the cells: nothing, character/factor, or numeric
  if(is.null(fill.string)) fill.type="none"
  else if (is.factor(data[[fill.string]]))
    fill.type <- "factor"
  else if (is.character(data[[fill.string]])){
    data[[fill.string]] <- as.factor(data[[fill.string]])
    fill.type <- "factor"
  } else {
    fill.type <- "num"
  }
  
  # Now get the fill values/length
  if(fill.type=="none") {
    fill.val <- rep(1, nrow(data))
    fill.n <- 1
    # Hack.  We need something to plot, call it .const
    form <- as.formula(paste(".const", form[[1]], deparse(form[[2]]), sep=""))
    data[['.const']] <- fill.val
  } else if(fill.type=="num"){
    fill.val <- data[[fill.string]]
  } else { # character/factor
    fill.val <- data[[fill.string]]
    fill.n <- nlevels(fill.val)
  }

  # Define fill colors and 'at' (if not given by the user)
  # at = # cut points for region colors
   if(fill.type=="none") {
    col.regions <- "transparent"
    at <- c(0.5,1.5)
  } else if(fill.type=="factor"){
    # If col.regions is a function, switch to default fill colors
    if(is.function(col.regions))
      col.regions <- c("#E6E6E6","#FFD9D9","#FFB2B2","#FFD7B2","#FDFFB2",
                       "#D9FFB2","#B2D6FF","#C2B2FF","#F0B2FF","#A6FFC9",
                       "#FF8C8C","#B2B2B2","#FFBD80","#BFFF80","#80BAFF",
                       "#9980FF","#E680FF","#D0D192","#59FF9C","#FFA24D",
                       "#FBFF4D","#4D9FFF","#704DFF","#DB4DFF","#808080",
                       "#9FFF40","#C9CC3D")
    # Handle named vectors for col.regions
    if(!is.null(names(col.regions))) {
      fill.levels <- levels(fill.val)
      matched_colors <- col.regions[fill.levels]
      # Check if all levels were matched
      if(any(is.na(matched_colors))) {
        missing_levels <- fill.levels[is.na(matched_colors)]
        warning("col.regions: Not all factor levels found in provided names. ",
                "Missing: ", paste(missing_levels, collapse=", "),
                ". Falling back to positional matching.")
        col.regions <- rep(col.regions, length=fill.n)
      } else {
        col.regions <- as.vector(matched_colors)
      }
    } else {
      col.regions <- rep(col.regions, length=fill.n)
    }
    at <- c((0:fill.n)+.5)
  } else if(fill.type=="num") {
    if(missing(at) && is.null(midpoint)){
        nbins <- 15
        if(is.function(col.regions)) col.regions <- col.regions(nbins)
        # Use lel = lattice:::extend.limits to move breakpoints past ends of fill.val
        zrng <- lel(range(as.numeric(fill.val), finite = TRUE))
        at <- seq(zrng[1], zrng[2], length.out = 16)
    }
    if(missing(at) && midpoint=="median"){ # default case for continuous data
      if(is.function(col.regions)) {
        nbins <- 15
        col.regions <- col.regions(nbins)
      } else {
        nbins <- length(col.regions)
      }
      med <- median(fill.val, na.rm=TRUE)
      radius <- max(max(fill.val, na.rm=TRUE)-med,
                    med-min(fill.val, na.rm=TRUE)) + .Machine$double.eps
      zrng <- lel(range(c(med-radius, med+radius)))
      brks <- seq(zrng[1], zrng[2], length.out = nbins+1)
      binno <- as.numeric(cut(fill.val, breaks=brks)) # bin number for each fill.val
      # select only 'col.regions' and 'at' values we actually need
      minbin <- min(binno, na.rm=TRUE); maxbin <- max(binno, na.rm=TRUE)
      col.regions <- col.regions[minbin:maxbin]
      at <- brks[minbin:(maxbin+1)]
    }
    if(missing(at) && midpoint=="midrange"){ # halfway between min & max
      if(is.function(col.regions)) {
        nbins <- 15
        col.regions <- col.regions(nbins)
      } else {
        nbins <- length(col.regions)
      }
      med <- median(range(fill.val, na.rm=TRUE))
      radius <- max(max(fill.val, na.rm=TRUE)-med,
                    med-min(fill.val, na.rm=TRUE)) + .Machine$double.eps
      zrng <- lel(range(c(med-radius, med+radius)))
      brks <- seq(zrng[1], zrng[2], length.out = nbins+1)
      binno <- as.numeric(cut(fill.val, breaks=brks)) # bin number for each fill.val
      # select only 'col.regions' and 'at' values we actually need
      minbin <- min(binno, na.rm=TRUE); maxbin <- max(binno, na.rm=TRUE)
      col.regions <- col.regions[minbin:maxbin]
      at <- brks[minbin:(maxbin+1)]
    }
    if(missing(at) && is.numeric(midpoint)){
      if(is.function(col.regions)) {
        nbins <- 15
        col.regions <- col.regions(nbins)
      } else {
        nbins <- length(col.regions)
      }
      radius <- max(max(fill.val, na.rm=TRUE)-midpoint,
                    midpoint-min(fill.val, na.rm=TRUE)) + .Machine$double.eps
      zrng <- lel(range(c(midpoint-radius, midpoint+radius)))
      brks <- seq(zrng[1], zrng[2], length.out = nbins+1)
      binno <- as.numeric(cut(fill.val, breaks=brks)) # bin number for each fill.val
      # select only col.regions and at we actually need
      minbin <- min(binno, na.rm=TRUE); maxbin <- max(binno, na.rm=TRUE)
      col.regions <- col.regions[minbin:maxbin]
      at <- brks[minbin:(maxbin+1)]
    }
    if(!missing(at)){
      # user specified 'at' and 'col.regions'
      nbins <- length(at)-1
      if(is.function(col.regions)) col.regions <- col.regions(nbins)
    }
    
  } # end fill.type

  # Text colors
  if(is.null(col.text))
    col.text <- c("black", "red3", "darkorange2", "chartreuse4",
                  "deepskyblue4", "blue", "purple4", "darkviolet", "maroon")

  # Change x/y from factor to numeric if needed.
  fac2num <- function(x) as.numeric(levels(x))[x]
  if(is.factor(data[[x.string]])) 
    data[[x.string]] <- fac2num(data[[x.string]])
  if(is.factor(data[[y.string]]))
    data[[y.string]] <- fac2num(data[[y.string]])

  # Resolve the (overloaded) 'ticks' argument into a show flag + per-axis breaks.
  # Done after x/y are numeric so "all" can enumerate the integer coordinates.
  tk <- .resolve_ticks(ticks, data[[x.string]], data[[y.string]])

  # If ticks are requested, add axis labels
  if (missing(xlab))
    xlab <- ifelse(tk$show, x.string, "")
  if (missing(ylab))
    ylab <- ifelse(tk$show, y.string, "")

  # Check for multiple values for each cell.
  if(is.null(ff$cond)) {
    # no factor for panels
    tt <- table(data[[x.string]], data[[y.string]])
  } else if(length(ff$cond) == 1L) {
    # one conditioning factor
    tt <- table(data[[x.string]], data[[y.string]], data[[ff$cond[1]]])
  } else if(length(ff$cond)==2L) {
    # two conditioning factors
    tt <- table(data[[x.string]], data[[y.string]], data[[ff$cond[1]]], data[[ff$cond[2]]])
  } else {
    message("Not checking for multiple data for each x/y/panel combination")
    tt <- NULL
  }
  if(any(tt>1))
    warning("There are multiple data for each x/y/panel combination")

  # lr = legend row count
  # lt = legend text strings
  lr <- 0
  lt <- NULL

  # should I add a legend line for dq ?
  
  if(has.out1){ # out1
    lr <- lr + 1
    lt <- c(lt, out1.string)
  }
  if(has.out2){ # out2
    lr <- lr + 1
    lt <- c(lt, out2.string)
  }
  if(has.out1 | has.out2) lr <- lr + 1 # blank line

  if(fill.type=="factor") { # fill
    lt.fill <- levels(fill.val)
    lr <- lr + 2 + fill.n
    lt <- c(lt, lt.fill)
  }

  if(has.num) { # number
    num.val <- factor(data[[num.string]])
    lt.num <- levels(num.val)
    num.n <- length(lt.num)
    lr <- lr + 2 + num.n
    lt <- c(lt, lt.num)
  }

  if(has.col) { # color
    col.val <- factor(data[[col.string]]) # In case it is numeric
    lt.col <- levels(col.val)
    col.n <- length(lt.col)
    lr <- lr + 2 + col.n
    lt <- c(lt, lt.col)
    # Handle named vectors for col.text
    if(!is.null(names(col.text))) {
      matched_colors <- col.text[lt.col]
      # Check if all levels were matched
      if(any(is.na(matched_colors))) {
        missing_levels <- lt.col[is.na(matched_colors)]
        warning("col.text: Not all factor levels found in provided names. ",
                "Missing: ", paste(missing_levels, collapse=", "),
                ". Falling back to positional matching.")
        if(length(col.text) < col.n) col.text <- rep(col.text, length=col.n)
      } else {
        col.text <- as.vector(matched_colors)
      }
    } else {
      if(length(col.text) < col.n) col.text <- rep(col.text, length=col.n)
    }
  } else {
    col.val <- rep(1, nrow(data)) # No color specified, use black by default
  }

  if(has.text) { # text
    text.val <- factor(data[[text.string]]) # In case it is not a factor
    lt.text <- levels(text.val)
    text.n <- length(lt.text)
    lr <- lr + 2 + text.n
    lt <- c(lt, lt.text)
  }

  # Set up short version of text
  if(has.text & is.null(text.levels)){
    if(shorten=='no' | shorten=='none' | (is.logical(shorten) && !shorten))
      text.levels <- lt.text
    else if (shorten=='abb')
      text.levels <- abbreviate(lt.text, 2, method='both')
    else if (shorten=='sub')
      text.levels <- substring(lt.text, 1, 3)
  }

  # We might not have a key, even though it was requested
  if (lr==0) show.key <- FALSE

  # In function call we use 'list' instead of 'gpar' because gpar is not
  # exported from grid, so now change the class for out1.gpar, out2.gpar
  #if(class(out1.gpar) != "gpar") class(out1.gpar) <- "gpar"
  class(out1.gpar) <- "gpar"
  #if(class(out2.gpar) != "gpar") class(out2.gpar) <- "gpar"
  class(out2.gpar) <- "gpar"
  
  # ----- Now we can actually set up the legend grobs -----
  if(show.key) {
    longstring <- lt[which.max(nchar(lt))]
    if(missing(key.cex)) {
      if(lr < 30) key.cex <- 1
      else if(lr < 40) key.cex <- .75
      else key.cex <- 0.5
    }

    foo <- frameGrob(layout = grid.layout(nrow = lr, ncol = 2,
                       heights = unit(rep(key.cex, lr), "lines"),
                       widths = unit(c(1,1), c("cm","strwidth"),
                         data=list(NULL, longstring))))

    offset <- 1

    if(has.out1){ # outline
      foo <- placeGrob(foo, linesGrob(x = unit(c(.2, .8), "npc"),
                                      y = unit(.5, "npc"),
                                      gp=out1.gpar),
                       row = offset, col = 1)
      foo <- placeGrob(foo, textGrob(label = out1.string, gp=gpar(cex=key.cex)),
                       row = offset, col = 2)
      offset <- offset + 1
    }
    if(has.out2){
      foo <- placeGrob(foo, linesGrob(x=c(.2,.8), y=.5, gp=out2.gpar),
                       row = offset, col = 1)
      foo <- placeGrob(foo, textGrob(label = out2.string, gp=gpar(cex=key.cex)),
                       row = offset, col = 2)
      offset <- offset + 1
    }
    if(has.out1 | has.out2) offset <- offset + 1 # blank line

    if(fill.type=='factor') {  # fill
      foo <- placeGrob(foo, textGrob(label = fill.string, gp=gpar(cex=key.cex)),
                       row = offset, col = 2)
      for(kk in 1:fill.n){
        foo <- placeGrob(foo, rectGrob(width = 0.6,
                                       gp = gpar(col="#FFFFCC",
                                         fill=col.regions[kk], cex=key.cex)),
                         row = offset + kk, col = 1)
        foo <- placeGrob(foo, textGrob(label = lt.fill[kk],
                                       gp=gpar(cex=key.cex)),
                         row = offset+kk, col = 2)
      }
      offset <- offset + 1 + fill.n + 1
    }

    if(has.num) { # number
      foo <- placeGrob(foo, textGrob(label = num.string, gp=gpar(cex=key.cex)),
                       row = offset, col = 2)
      for(kk in 1:num.n){
        foo <- placeGrob(foo, textGrob(label = kk, gp=gpar(cex=key.cex)),
                         row = offset + kk, col = 1)
        foo <- placeGrob(foo, textGrob(label = lt.num[kk], gp=gpar(cex=key.cex)),
                         row = offset + kk, col = 2)
      }
      offset <- offset + 1 + num.n + 1
    }

    if(has.col) { # color
      foo <- placeGrob(foo, textGrob(label = col.string, gp=gpar(cex=key.cex)),
                       row = offset, col = 2)
      for(kk in 1:col.n){
        foo <- placeGrob(foo, pointsGrob(.5,.5, pch=19, # 19 = solid circle
                                         gp=gpar(col=col.text[kk],
                                           cex=key.cex)),
                         row = offset + kk, col = 1)
        foo <- placeGrob(foo, textGrob(label = lt.col[kk], gp=gpar(cex=key.cex)),
                         row = offset + kk, col = 2)
      }
      offset <- offset + 1 + col.n + 1
    }

    if(has.text) { # text
      foo <- placeGrob(foo, textGrob(label = text.string, gp=gpar(cex=key.cex)),
                       row = offset, col = 2)
      for(kk in 1:text.n){
        foo <- placeGrob(foo, textGrob(label = text.levels[kk],
                                       gp=gpar(cex=key.cex)),
                         row = offset + kk, col = 1)
        foo <- placeGrob(foo, textGrob(label = lt.text[kk], gp=gpar(cex=key.cex)),
                         row = offset + kk, col = 2)
      }
      offset <- offset + 1 + text.n + 1
    }

  } else foo <- NULL

  # Cell text
  if(has.text) {
    cell.text <- text.levels[as.numeric(text.val)]
  } else if(has.num) {
    cell.text <- as.numeric(num.val)
  } else if(has.col) {
    cell.text <- rep("+", length=nrow(data))
  }
  
  # Data quality flag
  if(has.dq) {
    dq.val <- as.factor(data[[dq.string]])
    levels(dq.val) <- list("0"=c("0","G","Good"),
                           "1"=c("1","Q","Questionable"),
                           "2"=c("2","B","Bad","S","Suppressed"))
    dq.val <- as.numeric(as.character(dq.val))
  } else dq.val <- NULL
    
  out1.val <- if(has.out1) data[[out1.string]] else NULL
  out2.val <- if(has.out2) data[[out2.string]] else NULL

  # Assemble the axis scales from the resolved 'ticks'. A NULL break vector
  # (tk$x / tk$y) leaves that axis at lattice's default (pretty) breaks.
  scales.arg <- list(relation = "free", draw = tk$show)
  if(!is.null(tk$x)) scales.arg$x <- list(at = tk$x)
  if(!is.null(tk$y)) scales.arg$y <- list(at = tk$y)

  out <-
    levelplot(form,
              data=data,
              out1f=out1.val, out1g=out1.gpar,
              out2f=out2.val, out2g=out2.gpar,
              dq=dq.val,
              flip=flip,
              col.regions=col.regions,
              colorkey = if(fill.type=="num") TRUE else FALSE,
              as.table=TRUE,
              at=at,
              legend=if(show.key) list(left=list(fun=foo)) else list(),
              main=main,
              xlab=xlab,
              ylab=ylab,
              scales=scales.arg,
              prepanel = prepanel.desplot,
              panel=function(x, y, z, subscripts, groups, ...,
                             out1f, out1g, out2f, out2g, dq){
                # Fill the cells and outline
                panel.outlinelevelplot(x, y, z, subscripts, at, ...,
                                       out1f=out1f, out1g=out1g,
                                       out2f=out2f, out2g=out2g,
                                       dq=dq)

                # Numbers, colors, or text
                if(has.num|has.text|has.col)
                  panel.text(x[subscripts], y[subscripts],
                             cell.text[subscripts],
                             cex=cex,
                             col=col.text[as.numeric(col.val[subscripts])])
              },
              strip=strip.custom(par.strip.text=list(cex=strip.cex)), ...)

  # panel.border=FALSE: drop the panel box + axis lines that lattice draws by
  # default (the ggplot2 version does the same). Merge into any par.settings the
  # user passed through '...' rather than adding a second par.settings argument.
  if(!panel.border) {
    ps <- out$par.settings
    if(is.null(ps)) ps <- list()
    al <- ps$axis.line
    if(is.null(al)) al <- list()
    al$col <- "transparent"
    ps$axis.line <- al
    out <- update(out, par.settings = ps)
  }

  # Use 'update' for any other modifications
  #if(!show.key) out <- update(out, legend=list(left=NULL))

  return(out)
}

prepanel.desplot <- function (x, y, subscripts, flip, ...) {
  # based on lattice:::prepanel.default.levelplot
  pad <- lattice.getOption("axis.padding")$numeric

  # Note: x and y are NOT factors

  if (length(subscripts) > 0) {
    x <- x[subscripts]
    y <- y[subscripts]

    ux <- sort(unique(x[is.finite(x)]))
    if ((ulen <- length(ux)) < 2)
      #xlim <- ux + c(-1, 1)
      xlim <- ux + c(-0.5, 0.5)
    else {
      diffs <- diff(as.numeric(ux))[c(1, ulen - 1)]
      xlim <- c(ux[1] - diffs[1]/2, ux[ulen] + diffs[2]/2)
    }
    uy <- sort(unique(y[is.finite(y)]))
    if ((ulen <- length(uy)) < 2)
      #ylim <- uy + c(-1, 1)
      ylim <- uy + c(-0.5, 0.5)
    else {
      diffs <- diff(as.numeric(uy))[c(1, ulen - 1)]
      ylim <- c(uy[1] - diffs[1]/2, uy[ulen] + diffs[2]/2)
    }

    # This is returned
    ret <- list(xlim = lel(xlim, prop = -pad/(1 + 2 * pad)),
                ylim = lel(ylim, prop = -pad/(1 + 2 * pad)),
                dx = length(ux),
                dy = length(uy))
    if(flip) ret$ylim <- rev(ret$ylim)

    ret
  }
  else {
    # This is the value of the prepanel.null() function
    list(xlim = rep(NA_real_, 2), ylim = rep(NA_real_, 2), dx = NA_real_,
        dy = NA_real_)
  }
}

#' Panel Function for desplot
#'
#' This is a panel function for \code{desplot} which fills cells with
#' a background color and adds outlines around blocks of cells.
#'
#' It does not add the text labels, numbers, or colors.
#'
#' The rule for determining where to draw outlines is to compare the
#' levels of the factor used for outlining.  If bordering cells have
#' different levels of the factor, then a border is drawn.  'NA' values
#' are ignored (otherwise, too many lines would be drawn).
#'
#' The code works, but is probably overkill and has not been streamlined.
#' 
#' @param x Coordinates
#' 
#' @param y Coordinates
#' 
#' @param z Value for filling each cell.
#' 
#' @param subscripts For compatibility.
#' 
#' @param at Breakpoints for the colors.
#' 
#' @param ... Other
#' 
#' @param alpha.regions Transparency for fill colors. Not well tested.
#' 
#' @param out1f Factor to use for outlining (level 1).
#' 
#' @param out1g Factor to use for outlining (level 2).
#' 
#' @param out2f Graphics parameters to use for outlining.
#' 
#' @param out2g Graphics parameters to use for outlining.
#' 
#' @param dq Indicator of which cells should be flagged for data quality.
#' 
#' @export 
#' @references
#' None
#' 
panel.outlinelevelplot <- function(x, y, z, subscripts, at,
                                   ...,
                                   alpha.regions = 1,
                                   out1f, out1g, out2f, out2g,
                                   dq) {
  #  Derived from lattice::panel.levelplot
  dots=list(...)
  col.regions=dots$col.regions

  # parent function forces x,y to be numeric, not factors
  
  if (length(subscripts) == 0L) return()

  dq <- dq[subscripts]
  x <- x[subscripts]
  y <- y[subscripts]
  z <- as.numeric(z)
  zcol <- level.colors(z, at, col.regions, colors = TRUE)
  
  zlim <- range(z, finite = TRUE)
  z <- z[subscripts]
  zcol <- zcol[subscripts]
  
  ux <- sort(unique(x[!is.na(x)]))
  bx <- if (length(ux) > 1L) { # breakpoints
    c(3 * ux[1] - ux[2], ux[-length(ux)] + ux[-1],
      3 * ux[length(ux)] - ux[length(ux) - 1])/2
  } else ux + c(-0.5, 0.5)
  lx <- diff(bx) # lengths? I think this is same as rep(1, length(ux))
  cx <- (bx[-1] + bx[-length(bx)])/2  # centers
  
  uy <- sort(unique(y[!is.na(y)]))
  by <- if (length(uy) > 1) {
    c(3 * uy[1] - uy[2], uy[-length(uy)] + uy[-1],
      3 * uy[length(uy)] - uy[length(uy) - 1])/2
  } else uy + c(-0.5, 0.5)
  ly <- diff(by)
  cy <- (by[-1] + by[-length(by)])/2
  
  idx <- match(x, ux)
  idy <- match(y, uy)
  
  # Fill the cells with background color
  grid.rect(x = cx[idx], y = cy[idy],
            width=lx[idx],
            height = ly[idy],
            default.units = "native",
            gp = gpar(fill = zcol, lwd = 1e-05, 
                      col="transparent",
                      alpha = alpha.regions))

  # Data quality indicator
  gp=list(col="black", lwd=1)
  class(gp) <- "gpar"
  # draw diagonal line from lower-left to upper-right
  xd=x[dq >= 1L]
  yd=y[dq >= 1L]
  if(length(xd)>0) grid.segments(x0=xd-.5, y0=yd-.5,
                                 x1=xd+.5, y1=yd+.5, 
                                 default.units="native", gp=gp)
  # draw diagonal line from upper-left to lower-right
  xd=x[dq >= 2L]
  yd=y[dq >= 2L]
  if(length(xd)>0) grid.segments(x0=xd-.5, y0=yd+.5,
                                 x1=xd+.5, y1=yd-.5, 
                                 default.units="native", gp=gp)
  
  # Outline factor 1
  if(!is.null(out1f)){
    bb <- calc_borders(x, y, as.character(out1f[subscripts]))
    if(!is.null(bb) && nrow(bb)>0) {
      grid.segments(x0 = bb$x, y0=bb$y, x1=bb$xend, y1=bb$yend,
                    default.units="native", gp=out1g)
    }
  }

  # Outline factor 2
  if(!is.null(out2f)){
    bb <- calc_borders(x, y, as.character(out2f[subscripts]))
    if(!is.null(bb) && nrow(bb)>0){
      grid.segments(x0 = bb$x, y0=bb$y, x1=bb$xend, y1=bb$yend,
                    default.units="native", gp=out2g)
    }
  }
  
  return()
}

#' @noRd
.addLevels <- function(dat, xvar='x', yvar='y', locvar=NULL){
  # For each loc, we want x/y coords to be complete.
  # NO: 1,2,4.  YES: 1,2,3,4.
  # Add one NA datum for each missing x and each missing y
  # This does NOT completely fill in the rectangle (as needed by asreml)

  # Previous code assumed locvar was character/factor, but still worked
  # if locvar was numeric because R was coercing the data when assigning the
  # locvar to the new row.  BUT, if dat was a tibble, then the coercion was
  # not working.  The code below fixes that problem so that now locvar is
  # allowed to be numeric.  Also slightly cleaner.
  
  if( is.factor(dat[[xvar]]) | is.factor(dat[[yvar]]) )
    stop("FIXME: ", xvar, " or ", yvar, " are factors.")

  if(is.null(locvar)) {
    # If there is no location variable, we use "1" as the location level
    uniquelocs <- "1"
  } else {
    oloc <- dat[[locvar]]
    uniquelocs <- unique( oloc )
  }

  for(loc.i in uniquelocs){
    # Index of data rows for this loc.
    if(is.null(locvar)){
      ix <- 1:nrow(dat)
    } else {
      ix <- which(dat[[locvar]] == loc.i)
    }
    
    # Add rows with new x values that were missing.
    ux <- unique(dat[ix, xvar])
    newx <- setdiff(seq(from=min(ux), to=max(ux), by=1), ux)
    n.newx <- length(newx)
    if(n.newx > 0){
      ix.new <- nrow(dat) + 1:n.newx
      dat[ix.new, xvar] <- newx # R creates these rows on the fly
      if(!is.null(locvar))
        dat[ix.new, locvar] <- rep(loc.i, n.newx)
    }
    # Same for y.
    uy <- unique(dat[ix, yvar])
    newy <- setdiff(seq(from=min(uy), to=max(uy), by=1), uy)
    n.newy <- length(newy)
    if(n.newy > 0){
      ix.new <- nrow(dat) + 1:n.newy
      dat[ix.new, yvar] <- newy
      if(!is.null(locvar))
        dat[ix.new, locvar] <- rep(loc.i, n.newy)
    }
  }
  
  return(dat)
}

#' @noRd
# lel is a very simple version of lattice:::extend.limits
lel <- function (lim, prop = lattice.getOption("axis.padding")$numeric) {

  if (lim[1] == lim[2])
    lim + 0.5 * c(-1, 1)
  else {
    d <- diff(as.numeric(lim))
    lim + prop * d * c(-1, 1)
  }
}

# Test Script for Named Vector Color Support - Issue #10
library(desplot)

# Simple test data: 2 rows x 3 columns, 3 factor levels
test_data <- data.frame(
  row = rep(1:2, each=3),
  col = rep(1:3, times=2),
  treat = factor(rep(c("A", "B", "C"), length.out=6))
)

test_that("named colors", {
  # TEST 1: Named vector - forward order
  my_colors <- c("skyblue", "pink", "lightgreen")
  names(my_colors) <- c("A", "B", "C")
  desplot(test_data, treat ~ col*row, col.regions=my_colors, main="Test 1: Named forward", gg=FALSE)
  desplot(test_data, treat ~ col*row, col.regions=my_colors, main="Test 1: Named forward (gg)", gg=TRUE)
  
  # TEST 2: Named vector - reversed order (KEY TEST from issue #10)
  my_colors_rev <- c("skyblue", "pink", "lightgreen")
  names(my_colors_rev) <- c("C", "B", "A")  # REVERSED!
  desplot(test_data, treat ~ col*row, col.regions=my_colors_rev, main="Test 2: Named reversed", gg=FALSE)
  desplot(test_data, treat ~ col*row, col.regions=my_colors_rev, main="Test 2: Named reversed (gg)", gg=TRUE)
  
  # TEST 3: Partial names (should warn and fallback)
  partial_colors <- c("red", "blue")
  names(partial_colors) <- c("A", "B")  # Missing C
  expect_warning(desplot(test_data, treat ~ col*row, col.regions=partial_colors, main="Test 3: Partial names", gg=FALSE))
  expect_warning(desplot(test_data, treat ~ col*row, col.regions=partial_colors, main="Test 3: Partial names (gg)", gg=TRUE))
  
  # TEST 4: Extra names (should work, extras ignored)
  extra_colors <- c("purple", "orange", "brown", "yellow")
  names(extra_colors) <- c("A", "B", "C", "D")  # D doesn't exist
  desplot(test_data, treat ~ col*row, col.regions=extra_colors, main="Test 4: Extra names", gg=FALSE)
  desplot(test_data, treat ~ col*row, col.regions=extra_colors, main="Test 4: Extra names (gg)", gg=TRUE)
  
  # TEST 5: Unnamed vector (backward compatibility)
  unnamed_colors <- c("coral", "cyan", "gold")
  desplot(test_data, treat ~ col*row, col.regions=unnamed_colors, main="Test 5: Unnamed", gg=FALSE)
  desplot(test_data, treat ~ col*row, col.regions=unnamed_colors, main="Test 5: Unnamed (gg)", gg=TRUE)
  
  # TEST 6: Named col.text (outline colors)
  text_colors <- c("red", "blue", "green")
  names(text_colors) <- c("C", "B", "A")  # Reversed
  desplot(test_data, treat ~ col*row, col=treat, col.text=text_colors, main="Test 6: Named col.text", gg=FALSE)
  desplot(test_data, treat ~ col*row, col=treat, col.text=text_colors, main="Test 6: Named col.text (gg)", gg=TRUE)
})

## ----setup------------------------------------------------------------------------------
library("knitr")
knitr::opts_chunk$set(fig.align="center", fig.width=6, fig.height=6)
options(width=90)

## ----yates------------------------------------------------------------------------------
library(agridat)
library(desplot)
data(yates.oats)
desplot(yates.oats, block ~ col+row,
        col=nitro, text=gen, cex=1, out1=block,
        out2=gen, out2.gpar=list(col = "gray50", lwd = 1, lty = 1))

## ----ryder------------------------------------------------------------------------------
library(agridat)
library(desplot)
data(ryder.groundnut)
gnut <- ryder.groundnut
m1 <- lm(dry ~ block + gen, gnut) # Standard RCB model
gnut$res <- resid(m1)
desplot(gnut, res ~ col + row, text=gen, cex=1,
        main="ryder.groundnut residuals from RCB model")

## ----panelorder, eval=FALSE-------------------------------------------------------------
# library(agridat)
# library(desplot)
# data(besag.met)
# desplot(besag.met, yield~col*row|county, main="default county ordering")
# library(forcats)
# besag.met <- transform(besag.met,
#                        county=fct_relevel(county, c("C1","C3","C5")))
# desplot(besag.met, yield~col*row|county, main="custom county ordering")

---
title: "Plotting field maps with the desplot package"
author: "Kevin Wright"
date: "`r Sys.Date()`"
bibliography: desplot.bib
output:
  rmarkdown::html_vignette:
vignette: >
  %\VignetteIndexEntry{Plotting field maps with the desplot package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

# Abstract

This short note shows how to plot a field map from an agricultural experiment and why that may be useful.

# R setup

```{r setup}
library("knitr")
knitr::opts_chunk$set(fig.align="center", fig.width=6, fig.height=6)
options(width=90)
```

## Example 1

First, a plot of the experimental design of the oats data from @yates1935complex.
```{r yates}
library(agridat)
library(desplot)
data(yates.oats)
# Older versions of agridat used x/y here instead of col/row
if(is.element("x",names(yates.oats)))
   yates.oats <- transform(yates.oats, col=x, row=y)
desplot(yates.oats, block ~ col+row,
        col=nitro, text=gen, cex=1, out1=block,
        out2=gen, out2.gpar=list(col = "gray50", lwd = 1, lty = 1))
```

## Example 2

This next example is from @ryder1981field.  Fit an ordinary RCB model with fixed effects for `block` and `genotype`.  Plot a heatmap of the residuals.

```{r ryder}
library(agridat)
library(desplot)
data(ryder.groundnut)
gnut <- ryder.groundnut
m1 <- lm(dry ~ block + gen, gnut) # Standard RCB model
gnut$res <- resid(m1)
desplot(gnut, res ~ col + row, text=gen, cex=1,
        main="ryder.groundnut residuals from RCB model")
```
Note the largest positive/negative residuals are adjacent to each other,
perhaps caused by the original data values being swapped.  Checking with
experiment investigators (managers, data collectors, etc.) is recommended.


## References
	

---
title: "Plotting field maps with the desplot package"
author: "Kevin Wright"
date: "`r Sys.Date()`"
bibliography: desplot.bib
output:
  rmarkdown::html_vignette:
vignette: >
  %\VignetteIndexEntry{Plotting field maps with the desplot package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

# Abstract

This short note shows how to plot a field map from an agricultural experiment and why that may be useful.

# R setup

```{r setup}
library("knitr")
knitr::opts_chunk$set(fig.align="center", fig.width=6, fig.height=6)
options(width=90)
```

## Example 1

First, a plot of the experimental design of the oats data from @yates1935complex.
```{r yates}
library(agridat)
library(desplot)
data(yates.oats)
desplot(yates.oats, block ~ col+row,
        col=nitro, text=gen, cex=1, out1=block,
        out2=gen, out2.gpar=list(col = "gray50", lwd = 1, lty = 1))
```

## Example 2

This next example is from @ryder1981field.  Fit an ordinary RCB model with fixed effects for `block` and `genotype`.  Plot a heatmap of the residuals.

```{r ryder}
library(agridat)
library(desplot)
data(ryder.groundnut)
gnut <- ryder.groundnut
m1 <- lm(dry ~ block + gen, gnut) # Standard RCB model
gnut$res <- resid(m1)
desplot(gnut, res ~ col + row, text=gen, cex=1,
        main="ryder.groundnut residuals from RCB model")
```
Note the largest positive/negative residuals are adjacent to each other,
perhaps caused by the original data values being swapped.  Checking with
experiment investigators (managers, data collectors, etc.) is recommended.

# Infrequently asked questions

## How do I change the ordering of panels?

Make sure that the panel variable is a factor and then change the levels of the factor.

In the example below, the first three panels are set to the levels C1, C3, C5. The other levels remain in the same (relative) order.

```{r panelorder, eval=FALSE}
library(agridat)
library(desplot)
data(besag.met)
desplot(besag.met, yield~col*row|county, main="default county ordering")
library(forcats)
besag.met <- transform(besag.met,
                       county=fct_relevel(county, c("C1","C3","C5")))
desplot(besag.met, yield~col*row|county, main="custom county ordering")
```
## References
	

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/desplot.R
\name{RedGrayBlue}
\alias{RedGrayBlue}
\title{Function to create a Red-Gray-Blue palette}
\usage{
RedGrayBlue(n)
}
\arguments{
\item{n}{Number of colors to create}
}
\value{
A vector of n colors.
}
\description{
A function to create a Red-Gray-Blue palette.
}
\details{
Using gray instead of white allows missing values to appear as white
(actually, transparent).
}
\examples{
pie(rep(1,11), col=RedGrayBlue(11))
title("RedGrayBlue(11)")
}
\author{
Kevin Wright
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/desplot.R, R/ggdesplot.R
\name{desplot}
\alias{desplot}
\alias{ggdesplot}
\title{Plot the layout/data of a field experiment.}
\usage{
desplot(
  data,
  form = formula(NULL ~ x + y),
  num = NULL,
  num.string = NULL,
  col = NULL,
  col.string = NULL,
  text = NULL,
  text.string = NULL,
  out1 = NULL,
  out1.string = NULL,
  out2 = NULL,
  out2.string = NULL,
  dq = NULL,
  dq.string = NULL,
  col.regions = RedGrayBlue,
  col.text = NULL,
  text.levels = NULL,
  out1.gpar = list(col = "black", lwd = 3),
  out2.gpar = list(col = "yellow", lwd = 1, lty = 1),
  at,
  midpoint = "median",
  ticks = FALSE,
  panel.border = TRUE,
  flip = FALSE,
  main = NULL,
  xlab,
  ylab,
  shorten = "abb",
  show.key = TRUE,
  key.cex,
  cex = 0.4,
  strip.cex = 0.75,
  aspect = NULL,
  subset = TRUE,
  gg = FALSE,
  ...
)

ggdesplot(
  data,
  form = formula(NULL ~ x + y),
  num = NULL,
  num.string = NULL,
  col = NULL,
  col.string = NULL,
  text = NULL,
  text.string = NULL,
  out1 = NULL,
  out1.string = NULL,
  out2 = NULL,
  out2.string = NULL,
  dq = NULL,
  dq.string = NULL,
  col.regions = RedGrayBlue,
  col.text = NULL,
  text.levels = NULL,
  out1.gpar = list(col = "black", lwd = 3),
  out2.gpar = list(col = "yellow", lwd = 1, lty = 1),
  at,
  midpoint = "median",
  ticks = FALSE,
  panel.border = TRUE,
  flip = FALSE,
  main = NULL,
  xlab,
  ylab,
  shorten = "abb",
  show.key = TRUE,
  key.cex,
  cex = 0.4,
  strip.cex = 0.75,
  aspect = NULL,
  subset = TRUE,
  gg = FALSE,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{form}{A formula like \code{yield~x*y|location}. Note x,y are numeric.}

\item{num}{Bare name (no quotes) of the column of the data to use 
as a factor for number-coding the text in each cell.}

\item{num.string}{String name of the column of the data to use 
as a factor for number-coding the text in each cell.}

\item{col}{Bare name (no quotes) of the column of the data to use
for color-coding the text shown in each cell.}

\item{col.string}{String name of the column of the data to use
for color-coding the text shown in each cell.}

\item{text}{Bare name (no quotes) of the column of the data to use 
for the actual text shown in each cell.}

\item{text.string}{String name of the column of the data to use 
for the actual text shown in each cell.}

\item{out1}{Bare name (no quotes) of the column of the data to use 
for first-level outlining around blocks of cells.}

\item{out1.string}{String name of the column of the data to use 
for first-level outlining around blocks of cells.}

\item{out2}{Bare name (no quotes) of the column of the data to use 
for second-level outlining around blocks of cells.}

\item{out2.string}{String name of the column of the data to use 
for second-level outlining around blocks of cells.}

\item{dq}{Bare name (no quotes) of the column of the data to use 
for indicating bad data quality with diagonal lines.
This can either be a numeric vector or a factor/text. 
Cells with 1/"Q"/"Questionable" have one diagonal line.
Cells with 2/"B"/"Bad","S","Suppressed" have crossed diagonal lines.}

\item{dq.string}{String name of the column of the data to use 
for indicating bad data quality with diagonal lines.}

\item{col.regions}{Colors for the fill color of cells.}

\item{col.text}{Vector of colors for text strings.}

\item{text.levels}{Character strings to use instead of default 'levels'.}

\item{out1.gpar}{A list of graphics parameters for first-level outlining.
Can either be an ordinary \code{list()} or a call to \code{gpar()} from the
  \code{grid} package.}

\item{out2.gpar}{Graphics parameters for second-level of outlining.}

\item{at}{Breakpoints for the color ribbon.  Use this instead of 'zlim'.
Note: using 'at' causes 'midpoint' to be set to NULL.}

\item{midpoint}{Method to find midpoint of the color ribbon.
One of 'midrange', 'median (default), or a numeric value.}

\item{ticks}{Controls the axis ticks and labels. One of: \code{FALSE}
(default, no axes), \code{TRUE} (axes with the default "pretty" breaks),
\code{"all"} (a break at every integer coordinate, resolved separately for
each axis), or a list \code{list(x=, y=)} for explicit per-axis control where
each element is a numeric vector of breaks or \code{"all"}. A missing list
element leaves that axis at the default breaks.}

\item{panel.border}{If TRUE (default), draw the panel border and axis lines.
If FALSE, omit them for a cleaner field map.}

\item{flip}{If TRUE, vertically flip the image.}

\item{main}{Main title.}

\item{xlab}{Label for x axis.}

\item{ylab}{Label for y axis.}

\item{shorten}{Method for shortening text in the key, either 'abb', 'sub', 'no', or FALSE.}

\item{show.key}{If TRUE, show the key on the left side. (This is not the ribbon.)}

\item{key.cex}{Left legend cex.}

\item{cex}{Expansion factor for text/number in each cell.}

\item{strip.cex}{Strip cex.}

\item{aspect}{Aspect ratio.
To get a map of a field with a true aspect ratio include 'aspect=ylen/xlen'.
For lattice, 'ylen' is the vertical length of the field and 'xlen'
is the horizontal length of the field.
For ggplot2, 'ylen' is the vertical length of each cell/plot and 'xlen'
is the horizontal length of each plot.}

\item{subset}{An expression that evaluates to logical index vector for subsetting the data.}

\item{gg}{If TRUE, desplot() switches to ggdesplot().}

\item{...}{Other.}
}
\value{
A lattice or ggplot2 object
}
\description{
Use this function to plot the layout of a rectangular lattice
field experiment and also the observed data values.
}
\details{
To create the plot using lattice graphics: 
1. \code{desplot(...)}.

To create the plot using ggplot2 graphics, use one of the following:
1. \code{ggdesplot(...)}.
2. \code{desplot(..., gg=TRUE)}. 
3. \code{options(desplot.gg=TRUE); desplot(...)}. 
Method 3 is useful to modify all results from existing scripts.

The lattice version is complete, mature, and robust.
The ggplot2 version is incomplete.  The legend can only show colors, 
and some function arguments are ignored.
In general, lattice graphics are about 4-5 times faster than ggplot2 graphics.
Not all lattice parameters are passed down to \code{xyplot}, but it
is possible to make almost any change to the plot by assigning the
desplot object to a variable and then edit the object by hand or use
\code{update} to modify the object.  Then print it manually.  See the
first example below.

Use \code{col.regions} to specify fill colors.  This can either be a vector
of colors or a function that produces a vector of colors.  If the response
variable is a factor and \code{col.regions} is a \emph{function}, it will
be ignored and the cells are filled with default light-colored backgrounds
and a key is placed on the left.  If the response variable is
\emph{numeric}, the cells are colored according to \code{col.regions}, and
a ribbon key is placed on the right.

Use \code{shorten='abb'} (this is default) to shorten the cell text to 2
characters using the \code{abbreviate} function
Use \code{shorten='sub'} to use a 3-character substring.
Use \code{shorten='no'} or \code{shorten=FALSE} for no shortening.

Note that two sub-plots with identical levels of the split-plot factor can
be adjacent to each other by virtue of appearing in different whole-plots.
To correctly outline the split-plot factor, simply concatenate the
whole-plot factor and sub-plot factor together.

To call this function inside another function, you can hack like this:
vr <- "yield"; vx <- "x"; vy <- "y";
eval(parse(text=paste("desplot(", vr, "~", vx, "*", vy, ", data=yates.oats)")))
}
\examples{
if(require(agridat)){

# Show how to customize any feature.  Here: make the strips bigger.
data(besag.met)
d1 <- desplot(besag.met, 
              yield ~ col*row|county, 
              main="besag.met",
              out1=rep, out2=block, out2.gpar=list(col="white"), strip.cex=2)
d1 <- update(d1, par.settings = list(layout.heights=list(strip=2)))
print(d1)

data(yates.oats)
# Show experiment layout in true aspect
# Field width = 4 plots * 44 links = 176 links
# Field length = 18 plots * 28.4 links = 511 links
# With lattice, the aspect ratio is y/x for the entire field
desplot(yates.oats, 
        yield ~ col+row, 
        out1=block, out2=gen, aspect=511/176)
# With ggplot, the aspect ratio is y/x for each cell
ggdesplot(yates.oats, 
        yield ~ col+row, 
        out1=block, out2=gen, aspect=28.4/44)

desplot(yates.oats,
        block ~ col+row,
        col=nitro, text=gen, cex=1, out1=block,
        out2=gen, out2.gpar=list(col = "gray50", lwd = 1, lty = 1))

# Overloaded 'ticks' (a break at every integer) and the 'panel.border' switch
desplot(yates.oats, yield ~ col+row, ticks="all", panel.border=FALSE)

}
}
\references{
K. Ryder (1981).
Field plans: why the biometrician finds them useful.
\emph{Experimental Agriculture}, 17, 243--256.
}
\author{
Kevin Wright
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/desplot.R
\name{panel.outlinelevelplot}
\alias{panel.outlinelevelplot}
\title{Panel Function for desplot}
\usage{
panel.outlinelevelplot(
  x,
  y,
  z,
  subscripts,
  at,
  ...,
  alpha.regions = 1,
  out1f,
  out1g,
  out2f,
  out2g,
  dq
)
}
\arguments{
\item{x}{Coordinates}

\item{y}{Coordinates}

\item{z}{Value for filling each cell.}

\item{subscripts}{For compatibility.}

\item{at}{Breakpoints for the colors.}

\item{...}{Other}

\item{alpha.regions}{Transparency for fill colors. Not well tested.}

\item{out1f}{Factor to use for outlining (level 1).}

\item{out1g}{Factor to use for outlining (level 2).}

\item{out2f}{Graphics parameters to use for outlining.}

\item{out2g}{Graphics parameters to use for outlining.}

\item{dq}{Indicator of which cells should be flagged for data quality.}
}
\description{
This is a panel function for \code{desplot} which fills cells with
a background color and adds outlines around blocks of cells.
}
\details{
It does not add the text labels, numbers, or colors.

The rule for determining where to draw outlines is to compare the
levels of the factor used for outlining.  If bordering cells have
different levels of the factor, then a border is drawn.  'NA' values
are ignored (otherwise, too many lines would be drawn).

The code works, but is probably overkill and has not been streamlined.
}
\references{
None
}
